package cn.iocoder.yudao.module.product.service.coupon.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.apollo.core.event.OrderCouponUpdateEvent;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.redis.helper.RedisHelper;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerContacts.CustomerContactsDO;
import cn.iocoder.yudao.module.customer.service.customerContacts.CustomerContactsService;
import cn.iocoder.yudao.module.ecw.api.currency.CurrencyApi;
import cn.iocoder.yudao.module.ecw.api.currency.dto.ExchangeRateRespDTO;
import cn.iocoder.yudao.module.ecw.dal.dataobject.unit.UnitDO;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductDO;
import cn.iocoder.yudao.module.product.convert.coupon.CouponLineConvert;
import cn.iocoder.yudao.module.ecw.enums.CouponTypeEnum;
import cn.iocoder.yudao.module.product.dal.dataobject.coupon.*;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductPriceDO;
import cn.iocoder.yudao.module.product.dal.mysql.coupon.*;
import cn.iocoder.yudao.module.product.dal.mysql.product.ProductMapper;
import cn.iocoder.yudao.module.product.dto.*;
import cn.iocoder.yudao.module.product.param.ProdConditionParam;
import cn.iocoder.yudao.module.product.service.coupon.CouponCustomerService;
import cn.iocoder.yudao.module.product.service.coupon.CouponService;
import cn.iocoder.yudao.module.product.service.product.ProductPriceService;
import cn.iocoder.yudao.module.product.vo.coupon.*;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import lombok.AllArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;

import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.MessageFormat;
import java.util.*;
import java.util.stream.Collectors;

import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

import cn.iocoder.yudao.module.product.convert.coupon.CouponConvert;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.product.enums.ErrorCodeConstants.*;

/**
 * 优惠券信息 Service 实现类
 *
 * @author 系统管理员
 */
@AllArgsConstructor
@Service
@Validated
public class CouponServiceImpl extends AbstractService<CouponMapper, CouponDO> implements CouponService {

    private final CouponMapper couponMapper;
    private final CouponLineMapper lineMapper;
    private final CouponCustomerService couponCustomerService;
    private final CouponProdMapper prodMapper;
    private final CouponContentMapper contentMapper;
    private final RedisHelper redisHelper;
    private final ApplicationContext applicationContext;
    private final CustomerContactsService customerContactsService;
    private final ProductPriceService productPriceService;
    private final ProductMapper productMapper;
    private final CurrencyApi currencyApi;


    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createCoupon(CouponCreateReqVO createReqVO) {
//        if (Objects.nonNull(createReqVO.getEndTime())) {
//            // 结束时间默认取当天的最后1秒为结束时间
//            createReqVO.setEndTime(DateUtil.endOfDay(createReqVO.getEndTime()));
//        }
        if (Objects.nonNull(createReqVO.getEndTime()) && DateUtil.endOfDay(createReqVO.getEndTime()).compareTo(new Date()) < 0) {
            throw exception(COUPON_END_TIME_NOT_LT_CURRENT_TIME);
        } else {
            // 避免复制过期数据提交造成的有效状态错误，这里初始化有效状态
            createReqVO.setOverdueStatus(true);
        }
        if (Objects.equals(CouponTypeEnum.COUPON.getValue(), createReqVO.getType())) {
            if (null == createReqVO.getReduceAmount() || createReqVO.getReduceAmount().compareTo(BigDecimal.ZERO) < 0) {
                throw exception(COUPON_REDUCE_AMOUNT_ERROR);
            }
        } else {
            if (Objects.equals(CouponTypeEnum.SPECIAL_OFFER.getValue(), createReqVO.getType())) {
                // 特价类型必须设定商品
                if (Objects.isNull(createReqVO.getProdIds()) || CollectionUtil.isEmpty(createReqVO.getProdIds())) {
                    throw exception(COUPON_SPECIAL_OFFER_PROD_NOT_NULL);
                }
            }
            if (CollectionUtil.isEmpty(createReqVO.getDiscountDetailedVOs())) {
                throw exception(COUPON_DISCOUNT_DETAILED_NOT_NULL);
            } else {
                // 校验封装优惠清单
                verificationDiscountDetailed(createReqVO.getType(), createReqVO.getDiscountDetailedVOs());
                createReqVO.setDiscountDetailed(JSONObject.toJSONString(createReqVO.getDiscountDetailedVOs()));
            }
        }
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        // 插入
        CouponDO coupon = CouponConvert.INSTANCE.convert(createReqVO);
        Date now = new Date();
        coupon.setCreator(String.valueOf(userId));
        coupon.setUpdater(String.valueOf(userId));
        coupon.setCreateTime(now);
        coupon.setUpdateTime(now);
        if (StringUtils.isBlank(coupon.getDiscountDetailed())) {
            // 格式话字段，避免空字符串传入导致mysql的json类型字段报格式错误
            coupon.setDiscountDetailed(null);
        }
        couponMapper.insert(coupon);


        CouponContentDO contentDO = new CouponContentDO();
        contentDO.setContentZh(createReqVO.getContentZh());
        contentDO.setContentEn(createReqVO.getContentEn());
        contentDO.setCouponId(coupon.getCouponId());
        contentMapper.insert(contentDO);
        if (1 == createReqVO.getStatus() && createReqVO.getIsSiteContent()) {
            // TODO 获取站内信模板

        }

        // 为优惠劵绑定可用线路
        if (createReqVO.getSuitableLineType() != 0 && CollectionUtil.isNotEmpty(createReqVO.getLineIds())) {
            if (0 == createReqVO.getStatus()) {
                // 存入草稿的优惠劵，将用户ID列表保存到缓存
                redisHelper.set(MessageFormat.format("coupon:line:list:{0}", coupon.getCouponId()), JSONObject.toJSONString(createReqVO.getLineIds()));
            } else if (1 == createReqVO.getStatus()) {
                List<CouponLineDO> lineDOList = createReqVO.getLineIds().stream().map(lineVO -> {
                    if (createReqVO.getIsSiteContent()) {
                        // TODO 向发放优惠劵用户发送站内信

                    }
                    CouponLineDO lineDO = CouponLineConvert.INSTANCE.convert(lineVO);
                    lineDO.setCouponId(coupon.getCouponId());
                    return lineDO;
                }).collect(Collectors.toList());
                lineMapper.insertBatch(lineDOList);
                redisHelper.delete(MessageFormat.format("coupon:line:list:{0}", coupon.getCouponId()));
            }
        }
        // 为用户发放优惠劵
        if (createReqVO.getSuitableUserType() != 0 && CollectionUtil.isNotEmpty(createReqVO.getCustomerIds())) {
            if (0 == createReqVO.getStatus()) {
                // 存入草稿的优惠劵，将用户ID列表保存到缓存
                redisHelper.set(MessageFormat.format("coupon:customer:list:{0}", coupon.getCouponId()), JSONObject.toJSONString(createReqVO.getCustomerIds()));
            } else if (1 == createReqVO.getStatus()) {
                List<CouponCustomerDO> customerDOList = createReqVO.getCustomerIds().stream().map(customerId -> {
                    if (createReqVO.getIsSiteContent()) {
                        // TODO 向发放优惠劵用户发送站内信

                    }
                    CouponCustomerDO customerDO = new CouponCustomerDO();
                    customerDO.setCouponId(coupon.getCouponId());
                    customerDO.setCustomerId(customerId);
                    if (createReqVO.getType() == 1) {
                        // 优惠劵发放优先绑定会员id
                        CustomerContactsDO contactsDO = customerContactsService.getCustomerContacts(customerId);
                        if (Objects.isNull(contactsDO)) {
                            return null;
                        }
                        customerDO.setUserId(contactsDO.getUserid());
                    }
                    customerDO.setStatus(1);
                    customerDO.setReceiveTime(new Date());
                    customerDO.setUserStartTime(coupon.getStartTime());
                    customerDO.setUserEndTime(coupon.getEndTime());
                    return customerDO;
                }).filter(Objects::nonNull).collect(Collectors.toList());
                couponCustomerService.saveBatch(customerDOList);
                redisHelper.delete(MessageFormat.format("coupon:customer:list:{0}", coupon.getCouponId()));
            }
        }

        // 为优惠劵绑定可用商品
        if (createReqVO.getSuitableProdType() != 0 && CollectionUtil.isNotEmpty(createReqVO.getProdIds())) {
            if (0 == createReqVO.getStatus()) {
                // 存入草稿的优惠劵，将产品ID列表保存到缓存
                redisHelper.set(MessageFormat.format("coupon:prod:list:{0}", coupon.getCouponId()), JSONObject.toJSONString(createReqVO.getProdIds()));
            } else if (1 == createReqVO.getStatus()) {
                List<CouponProdDO> prodDOList = createReqVO.getProdIds().stream().map(prodId -> {
                    CouponProdDO prodDO = new CouponProdDO();
                    prodDO.setCouponId(coupon.getCouponId());
                    prodDO.setProdId(prodId);
                    return prodDO;
                }).collect(Collectors.toList());
                prodMapper.insertBatch(prodDOList);
                redisHelper.delete(MessageFormat.format("coupon:prod:list:{0}", coupon.getCouponId()));
            }
        }
        if (coupon.getStatus() == 1 && coupon.getStartTime().compareTo(new Date()) < 0) {
            // TODO 当发布的优惠活动开始时间小于当前时间，则需要将开始时间与当前时间入仓的订单进行重新计价，以保证这段时间的入仓订单能够适配到该优惠活动
            applicationContext.publishEvent(new OrderCouponUpdateEvent(coupon.getTitleZh(), coupon.getTitleEn(), coupon.getStartTime()));
        }
        // 返回
        return coupon.getCouponId();
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateCoupon(CouponUpdateReqVO updateReqVO) {
//        if (Objects.nonNull(updateReqVO.getEndTime())) {
//            // 结束时间默认取当天的最后1秒为结束时间
//            updateReqVO.setEndTime(DateUtil.endOfDay(updateReqVO.getEndTime()));
//        }
        if (Objects.nonNull(updateReqVO.getEndTime()) && DateUtil.endOfDay(updateReqVO.getEndTime()).compareTo(new Date()) < 0) {
            throw exception(COUPON_END_TIME_NOT_LT_CURRENT_TIME);
        } else {
            // 避免复制过期数据提交造成的有效状态错误，这里初始化有效状态
            updateReqVO.setOverdueStatus(true);
        }
        // 校验存在
        CouponDO couponDO = couponMapper.selectById(updateReqVO.getCouponId());
        if (couponDO == null) {
            throw exception(COUPON_NOT_EXISTS);
        }
        if (1 == couponDO.getStatus()) {
            throw exception(COUPON_PUBLISHED);
        }
        if (Objects.equals(CouponTypeEnum.COUPON.getValue(), updateReqVO.getType())) {
            if (null == updateReqVO.getReduceAmount() || updateReqVO.getReduceAmount().compareTo(BigDecimal.ZERO) < 0) {
                throw exception(COUPON_REDUCE_AMOUNT_ERROR);
            }
        } else {
            if (CollectionUtil.isEmpty(updateReqVO.getDiscountDetailedVOs())) {
                throw exception(COUPON_DISCOUNT_DETAILED_NOT_NULL);
            } else {
                // 校验封装优惠清单
                verificationDiscountDetailed(updateReqVO.getType(), updateReqVO.getDiscountDetailedVOs());
                updateReqVO.setDiscountDetailed(JSONObject.toJSONString(updateReqVO.getDiscountDetailedVOs()));
            }
        }
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        CouponDO updateObj = CouponConvert.INSTANCE.convert(updateReqVO);

        Date now = new Date();
        updateObj.setUpdater(String.valueOf(userId));
        updateObj.setUpdateTime(now);

        couponMapper.updateById(updateObj);


        CouponContentDO contentDO = contentMapper.selectOne(new LambdaQueryWrapperX<CouponContentDO>()
                .eq(CouponContentDO::getCouponId, updateObj.getCouponId()).last("limit 1"));
        if (contentDO == null) {
            contentDO = new CouponContentDO();
        }
        contentDO.setContentZh(updateReqVO.getContentZh());
        contentDO.setContentEn(updateReqVO.getContentEn());
        contentDO.setCouponId(updateObj.getCouponId());
        if (null == contentDO.getCouponContentId()) {
            contentMapper.insert(contentDO);
        } else {
            contentMapper.updateById(contentDO);
        }

        if (1 == updateReqVO.getStatus() && updateReqVO.getIsSiteContent()) {
            // TODO 获取站内信模板1 == updateReqVO.getStatus()

        }
        // 为优惠劵绑定可用线路
        if (updateReqVO.getSuitableLineType() != 0 && CollectionUtil.isNotEmpty(updateReqVO.getLineIds())) {
            if (0 == updateReqVO.getStatus()) {
                // 存入草稿的优惠劵，将用户ID列表保存到缓存
                redisHelper.set(MessageFormat.format("coupon:line:list:{0}", updateObj.getCouponId()), JSONObject.toJSONString(updateReqVO.getLineIds()));
            } else if (1 == updateReqVO.getStatus()) {
                List<CouponLineDO> lineDOList = updateReqVO.getLineIds().stream().map(lineVO -> {
                    if (updateReqVO.getIsSiteContent()) {
                        // TODO 向发放优惠劵用户发送站内信

                    }
                    CouponLineDO lineDO = CouponLineConvert.INSTANCE.convert(lineVO);
                    lineDO.setCouponId(updateObj.getCouponId());
                    return lineDO;
                }).collect(Collectors.toList());
                lineMapper.insertBatch(lineDOList);
                redisHelper.delete(MessageFormat.format("coupon:line:list:{0}", updateObj.getCouponId()));
            }
        } else {
            redisHelper.delete(MessageFormat.format("coupon:line:list:{0}", updateObj.getCouponId()));
        }
        // 为用户发放优惠劵
        if (updateReqVO.getSuitableUserType() != 0 && CollectionUtil.isNotEmpty(updateReqVO.getCustomerIds())) {
            if (0 == updateReqVO.getStatus()) {
                // 存入草稿的优惠劵，将用户ID列表保存到缓存
                redisHelper.set(MessageFormat.format("coupon:customer:list:{0}", updateObj.getCouponId()), JSONObject.toJSONString(updateReqVO.getCustomerIds()));
            } else if (1 == updateReqVO.getStatus()) {
                List<CouponCustomerDO> customerDOList = updateReqVO.getCustomerIds().stream().map(customerId -> {
                    if (updateReqVO.getIsSiteContent()) {
                        // TODO 向发放优惠劵用户发送站内信

                    }
                    CouponCustomerDO customerDO = new CouponCustomerDO();
                    customerDO.setCouponId(updateObj.getCouponId());
                    customerDO.setCustomerId(customerId);
                    customerDO.setStatus(1);
                    customerDO.setReceiveTime(new Date());
                    customerDO.setUserStartTime(updateObj.getStartTime());
                    customerDO.setUserEndTime(updateObj.getEndTime());
                    return customerDO;
                }).collect(Collectors.toList());
                couponCustomerService.saveBatch(customerDOList);
                redisHelper.delete(MessageFormat.format("coupon:customer:list:{0}", updateObj.getCouponId()));
            }
        } else {
            redisHelper.delete(MessageFormat.format("coupon:customer:list:{0}", updateObj.getCouponId()));
        }

        // 为优惠劵绑定可用商品
        if (updateReqVO.getSuitableProdType() != 0 && CollectionUtil.isNotEmpty(updateReqVO.getProdIds())) {
            if (0 == updateReqVO.getStatus()) {
                // 存入草稿的优惠劵，将产品ID列表保存到缓存
                redisHelper.set(MessageFormat.format("coupon:prod:list:{0}", updateObj.getCouponId()), JSONObject.toJSONString(updateReqVO.getProdIds()));
            } else if (1 == updateReqVO.getStatus()) {
                List<CouponProdDO> prodDOList = updateReqVO.getProdIds().stream().map(prodId -> {
                    CouponProdDO prodDO = new CouponProdDO();
                    prodDO.setCouponId(updateObj.getCouponId());
                    prodDO.setProdId(prodId);
                    return prodDO;
                }).collect(Collectors.toList());
                prodMapper.insertBatch(prodDOList);
                redisHelper.delete(MessageFormat.format("coupon:prod:list:{0}", updateObj.getCouponId()));
            }
        } else {
            redisHelper.delete(MessageFormat.format("coupon:prod:list:{0}", updateObj.getCouponId()));
        }
        if (updateObj.getStatus() == 1 && updateObj.getStartTime().compareTo(new Date()) < 0) {
            // TODO 当发布的优惠活动开始时间小于当前时间，则需要将开始时间与当前时间入仓的订单进行重新计价，以保证这段时间的入仓订单能够适配到该优惠活动
            applicationContext.publishEvent(new OrderCouponUpdateEvent(updateObj.getTitleZh(), updateObj.getTitleEn(), updateObj.getStartTime()));
        }
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateCouponEndTime(Long couponId, Date endTime, boolean isValidity) {
        CouponDO couponDO = couponMapper.selectById(couponId);
        if (Objects.isNull(couponDO)) {
            throw exception(COUPON_NOT_EXISTS);
        }
        if (isValidity && Objects.isNull(endTime)) {
            throw exception(COUPON_END_TIME_NOT_NULL);
        }
//        if (Objects.nonNull(endTime)) {
//            // 结束时间默认取当天的最后1秒为结束时间
//            endTime = DateUtil.endOfDay(endTime);
//        }
        if (Objects.nonNull(couponDO.getEndTime()) && DateUtil.endOfDay(couponDO.getEndTime()).compareTo(new Date()) < 0) {
            if (DateUtil.endOfDay(endTime).compareTo(new Date()) < 0) {
                throw exception(COUPON_DELAY_TIME_MUST_GT_CURRENT_TIME);
            }
        }
        if (isValidity && Objects.nonNull(couponDO.getEndTime()) && couponDO.getEndTime().compareTo(endTime) > 0) {
            throw exception(COUPON_EXTENSION_TIME_NOT_LT_ORIGINAL_END_TIME);
        }
        if (isValidity && Objects.isNull(couponDO.getEndTime()) && endTime.compareTo(new Date()) < 0) {
            throw exception(COUPON_DELAY_TIME_MUST_GT_CURRENT_TIME);
        }
        this.update(new LambdaUpdateWrapper<CouponDO>()
                .set(CouponDO::getEndTime, isValidity ? endTime : null)
                .set(CouponDO::getOverdueStatus, 1)
                .eq(CouponDO::getCouponId, couponId));
        couponCustomerService.update(new LambdaUpdateWrapper<CouponCustomerDO>().set(CouponCustomerDO::getUserEndTime, isValidity ? endTime : null).eq(CouponCustomerDO::getCouponId, couponId));
    }

    @Override
    public void updateCouponPutonStatus(Long couponId, Integer putonStatus) {
        CouponDO couponDO = couponMapper.selectById(couponId);
        if (Objects.isNull(couponDO)) {
            throw exception(COUPON_NOT_EXISTS);
        }
        if (!Objects.equals(putonStatus, couponDO.getPutonStatus())) {
            this.update(new LambdaUpdateWrapper<CouponDO>()
                    .set(CouponDO::getPutonStatus, putonStatus)
                    .eq(CouponDO::getCouponId, couponId));
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteCoupon(Long id) {
        boolean b = this.update(new LambdaUpdateWrapper<CouponDO>().set(CouponDO::getDeleted, 1).eq(CouponDO::getCouponId, id));
        // 校验存在
        if (!b) {
            throw exception(COUPON_NOT_EXISTS);
        }
        couponCustomerService.update(new LambdaUpdateWrapper<CouponCustomerDO>().set(CouponCustomerDO::getIsDelete, true).eq(CouponCustomerDO::getCouponId, id));
    }


    @Override
    public CouponDO getCoupon(Long id) {
        return couponMapper.selectById(id);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void registerGiveCoupon(Long userId) {
        // 获取所有注册即送的优惠劵信息
        List<CouponDO> couponDOList = couponMapper.selectList(new LambdaQueryWrapper<CouponDO>()
                .eq(CouponDO::getSuitableUserType, 3));
        List<CouponCustomerDO> couponCustomerDOList = couponDOList.stream().map(couponDO -> {
            if (couponDO.getIsSiteContent()) {
                // TODO 向发放优惠劵用户发送站内信

            }
            CouponCustomerDO customerDO = new CouponCustomerDO();
            customerDO.setCouponId(couponDO.getCouponId());
            customerDO.setUserId(userId);
            customerDO.setStatus(1);
            customerDO.setReceiveTime(new Date());
            customerDO.setUserStartTime(couponDO.getStartTime());
            customerDO.setUserEndTime(couponDO.getEndTime());
            return customerDO;
        }).collect(Collectors.toList());
        couponCustomerService.saveBatch(couponCustomerDOList);
    }

    @Override
    public CouponUserAvailableDto getCouponUserAvailableDto(Integer orderType, Integer customsType, Integer isControl, Long userId,
                                                            Long consignorCustomerId, Long consigneeCustomerId,
                                                            List<ProdConditionParam> prodConditionParamList,
                                                            Integer transportId, Long lineId, Long channelId) {
        CouponUserAvailableDto userAvailableDto = new CouponUserAvailableDto();
        if (orderType == null || orderType == 0) {
            orderType = 1; // 普货
        }
        if (CollectionUtil.isEmpty(prodConditionParamList)) {
            throw exception(PROD_PARAM_NOT_NULL);
        }
//        CalculationCostResultDto dto = prodCostCalculation.calculationProdCost(prodConditionParamList, null, lineId, channelId);
//        List<ProdCostDto> prodCostDtoList = dto.getProdCostDtoList();
//        for (ProdConditionParam prodParam : prodConditionParamList) {
//            // 根据商品ID与线路ID获取商品运费与清关费的单价
//            ProductPriceDO productPriceDO = productPriceService.getOne(new LambdaQueryWrapper<ProductPriceDO>()
//                    .eq(ProductPriceDO::getProductId, prodParam.getProdId())
//                    .eq(ProductPriceDO::getWarehouseLineId, lineId)
//                    .and(we -> we.le(ProductPriceDO::getValidateStartDate, DateUtil.date())
//                            .or()
//                            .isNull(ProductPriceDO::getValidateStartDate))
//                    .and(we -> we.ge(ProductPriceDO::getValidateEndDate, DateUtil.date())
//                            .or()
//                            .isNull(ProductPriceDO::getValidateEndDate))
//                    .eq(null != channelId && 0 != channelId, ProductPriceDO::getShippingChannelId, channelId)
//                    .eq(ProductPriceDO::getDeleted, false)
//                    .last("limit 1"));
//        }

//        CostDto costDto = dto.getCostDto();
//        if (Objects.isNull(costDto)) {
//            throw exception(COST_CALCULATION_ERROR);
//        }
        // TODO 暂时不考虑优惠劵业务 优惠劵列表
//        List<CouponAvailableDto> couponDtoList = couponMapper.getCouponAvailableDto(userId, consignorCustomerContactsId);
//        if (CollectionUtil.isNotEmpty(couponDtoList)) {
//            List<CouponAvailableDto> dtoList = this.availableCouponAdd(costDto, couponDtoList);
//            userAvailableDto.setCouponAvailableDtoList(dtoList);
//        }
        List<CouponAvailableGroupDto> groupDtoList = new ArrayList<>();
        for (ProdConditionParam prodParam : prodConditionParamList) {
            // 根据商品ID与线路ID获取商品运费与清关费的单价
            this.coverPreferentialConditions(lineId, channelId, groupDtoList, prodParam);

        }
        // 优惠活动列表
        List<CouponAvailableDto> activityDtoList = couponMapper.getActivityCouponAvailableDto(orderType, customsType, isControl, userId,
                consignorCustomerId, consigneeCustomerId, prodConditionParamList, lineId, channelId, DateUtil.beginOfDay(new Date()));
        for (CouponAvailableDto couponAvailableDto : activityDtoList) {
            // 获取优惠活动使用条件
            List<DiscountDetailedVO> detailedVOList = couponAvailableDto.getDiscountDetailedVOList();
            if (CollectionUtil.isEmpty(detailedVOList)) {
                // 活动优惠条件清单为空，则忽略此优惠
                continue;
            }
            List<ProductDO> productDOList = couponAvailableDto.getProdList();
            if (CollectionUtil.isNotEmpty(productDOList)) {
                // 有指定商品条件
                List<CouponAvailableGroupDto> validProdCostDtoList = new ArrayList<>();
                for (CouponAvailableGroupDto groupDto : groupDtoList) {
                    long count = productDOList.stream().filter(productDO -> Objects.equals(productDO.getId(), groupDto.getProdId())).count();
                    if (count > 0) {
                        if (Arrays.stream((couponAvailableDto.getBrand().split(StrUtil.COMMA))).map(Integer::valueOf).collect(Collectors.toList()).contains(groupDto.getBrandType())) {
                            validProdCostDtoList.add(groupDto);
                        }
                    }
                }

                for (CouponAvailableGroupDto groupDto : validProdCostDtoList) {
                    // 类型 1:优惠卷 2:金额-满减 3:方数-满减 4 折扣 5 优惠 6 特价 7 区间价
                    this.availableActivityCouponAdd(couponAvailableDto, null, detailedVOList, groupDto);

                }
            } else {
                for (CouponAvailableGroupDto groupDto : groupDtoList) {
                    // 类型 1:优惠卷 2:金额-满减 3:方数-满减 4 折扣 5 优惠 6 特价 7 区间价
                    if (Arrays.stream((couponAvailableDto.getBrand().split(StrUtil.COMMA))).map(Integer::valueOf).collect(Collectors.toList()).contains(groupDto.getBrandType())) {
                        availableActivityCouponAdd(couponAvailableDto, null, detailedVOList, groupDto);
                    }

                }
            }
        }
        if (CollectionUtil.isNotEmpty(groupDtoList)) {
            for (CouponAvailableGroupDto groupDto : groupDtoList) {
                if (CollectionUtil.isNotEmpty(groupDto.getCouponAvailableDtoList())) {
                    groupDto.setCouponAvailableDtoList(this.comparingReduceAmount(groupDto.getCouponAvailableDtoList()));
                }
            }
            userAvailableDto.setCouponAvailableGroupDtoList(groupDtoList);
        }

        return userAvailableDto;
    }


    @Override
    public void getCouponUserAvailableDto(Integer transportId, Integer orderType, Integer customsType, Integer isControl, Long userId,
                                          Long consignorContactsCustomerId, Long consigneeContactsCustomerId, Long lineId,
                                          Long channelId, List<ProdConditionParam> paramList, Integer rmbCurrency, Map<Integer, UnitDO> unitDOMap) {
        // 优惠活动列表
        List<CouponAvailableDto> activityDtoList = couponMapper.getActivityCouponAvailableDto(orderType,
                customsType, isControl, userId, consignorContactsCustomerId, consigneeContactsCustomerId,
                paramList, lineId, channelId, DateUtil.beginOfDay(new Date()));

        // 根据订单条件匹配每个订单项的可用优惠活动
        for (CouponAvailableDto couponAvailableDto : activityDtoList) {
            // 获取优惠活动使用条件
            List<DiscountDetailedVO> detailedVOList = couponAvailableDto.getDiscountDetailedVOList();
            if (CollectionUtil.isEmpty(detailedVOList)) {
                // 活动优惠条件清单为空，则忽略此优惠
                continue;
            }
            // 收费值
            BigDecimal couponTotalChargeVolume = BigDecimal.ZERO;
            BigDecimal couponTotalChargeWeight = BigDecimal.ZERO;
            BigDecimal couponTotalChargeQuantity = BigDecimal.ZERO;
            // 实际值
            BigDecimal couponTotalVolume = BigDecimal.ZERO;
            BigDecimal couponTotalWeight = BigDecimal.ZERO;
            BigDecimal couponTotalQuantity = BigDecimal.ZERO;
            for (ProdConditionParam prodParam : paramList) {
                List<CouponAvailableDto> prodAvailableDtoList = prodParam.getActivityDtoList();
                List<ProductDO> productDOList = couponAvailableDto.getProdList();
                if (CollectionUtil.isNotEmpty(productDOList)) {
                    // 有指定商品条件
                    long count = productDOList.stream().filter(productDO -> Objects.equals(productDO.getId(), prodParam.getProdId())).count();
                    if (count > 0) {
                        if (Arrays.stream((couponAvailableDto.getBrand().split(StrUtil.COMMA))).map(Integer::valueOf).collect(Collectors.toList()).contains(prodParam.getBrandType())) {
                            // 类型 1:优惠卷 2:金额-满减 3:方数-满减 4 折扣 5 优惠 6 特价 7 区间价
                            if (Objects.isNull(prodAvailableDtoList)) {
                                prodAvailableDtoList = new ArrayList<>();
                            }
                            prodAvailableDtoList.add(couponAvailableDto);

                            couponTotalChargeVolume = couponTotalChargeVolume.add(Objects.nonNull(prodParam.getChargeVolume()) && prodParam.getChargeVolume().compareTo(BigDecimal.ZERO) > 0 ? prodParam.getChargeVolume() : prodParam.getVolume());
                            couponTotalChargeWeight = couponTotalChargeWeight.add(Objects.nonNull(prodParam.getChargeWeight()) && prodParam.getChargeWeight().compareTo(BigDecimal.ZERO) > 0 ? prodParam.getChargeWeight() : prodParam.getWeight());
                            couponTotalChargeQuantity = couponTotalChargeQuantity.add(Objects.nonNull(prodParam.getChargeQuantity()) && prodParam.getChargeQuantity().compareTo(BigDecimal.ZERO) > 0 ? prodParam.getChargeQuantity() : (Objects.nonNull(prodParam.getQuantity()) ? new BigDecimal(prodParam.getQuantity()) : BigDecimal.ZERO));

                            couponTotalVolume = couponTotalVolume.add(prodParam.getVolume());
                            couponTotalWeight = couponTotalWeight.add(prodParam.getWeight());
                            couponTotalQuantity = couponTotalQuantity.add((Objects.nonNull(prodParam.getQuantity()) ? new BigDecimal(prodParam.getQuantity()) : BigDecimal.ZERO));
                        }
                    }
                } else {
                    // 类型 1:优惠卷 2:金额-满减 3:方数-满减 4 折扣 5 优惠 6 特价 7 区间价
                    if (Arrays.stream((couponAvailableDto.getBrand().split(StrUtil.COMMA))).map(Integer::valueOf).collect(Collectors.toList()).contains(prodParam.getBrandType())) {
                        if (Objects.isNull(prodAvailableDtoList)) {
                            prodAvailableDtoList = new ArrayList<>();
                        }
                        prodAvailableDtoList.add(couponAvailableDto);

                        couponTotalChargeVolume = couponTotalChargeVolume.add(Objects.nonNull(prodParam.getChargeVolume()) && prodParam.getChargeVolume().compareTo(BigDecimal.ZERO) > 0 ? prodParam.getChargeVolume() : prodParam.getVolume());
                        couponTotalChargeWeight = couponTotalChargeWeight.add(Objects.nonNull(prodParam.getChargeWeight()) && prodParam.getChargeWeight().compareTo(BigDecimal.ZERO) > 0 ? prodParam.getChargeWeight() : prodParam.getWeight());
                        couponTotalChargeQuantity = couponTotalChargeQuantity.add(Objects.nonNull(prodParam.getChargeQuantity()) && prodParam.getChargeQuantity().compareTo(BigDecimal.ZERO) > 0 ? prodParam.getChargeQuantity() : (Objects.nonNull(prodParam.getQuantity()) ? new BigDecimal(prodParam.getQuantity()) : BigDecimal.ZERO));

                        couponTotalVolume = couponTotalVolume.add(prodParam.getVolume());
                        couponTotalWeight = couponTotalWeight.add(prodParam.getWeight());
                        couponTotalQuantity = couponTotalQuantity.add((Objects.nonNull(prodParam.getQuantity()) ? new BigDecimal(prodParam.getQuantity()) : BigDecimal.ZERO));
                    }

                }
                prodParam.setActivityDtoList(prodAvailableDtoList);
            }
            couponAvailableDto.setCouponTotalChargeVolume(couponTotalChargeVolume);
            couponAvailableDto.setCouponTotalChargeWeight(couponTotalChargeWeight);
            couponAvailableDto.setCouponTotalChargeQuantity(couponTotalChargeQuantity);
            couponAvailableDto.setCouponTotalVolume(couponTotalVolume);
            couponAvailableDto.setCouponTotalWeight(couponTotalWeight);
            couponAvailableDto.setCouponTotalQuantity(couponTotalQuantity);
        }
        // 计算相同优惠下的商品项数据累加值
        Map<Long, Map<String, BigDecimal>> couponConditionMap = new HashMap<>();
        for (CouponAvailableDto availableDto : activityDtoList) {
            Map<String, BigDecimal> couponTotalChargeMap = new HashMap<>(6);
            couponTotalChargeMap.put("couponTotalChargeVolume", availableDto.getCouponTotalChargeVolume());
            couponTotalChargeMap.put("couponTotalChargeWeight", availableDto.getCouponTotalChargeWeight());
            couponTotalChargeMap.put("couponTotalChargeQuantity", availableDto.getCouponTotalChargeQuantity());
            couponTotalChargeMap.put("couponTotalVolume", availableDto.getCouponTotalVolume());
            couponTotalChargeMap.put("couponTotalWeight", availableDto.getCouponTotalWeight());
            couponTotalChargeMap.put("couponTotalQuantity", availableDto.getCouponTotalQuantity());
            couponConditionMap.put(availableDto.getCouponId(), couponTotalChargeMap);
        }
        for (ProdConditionParam prodParam : paramList) {
            UnitDO unitDO = prodParam.getUnitDO();
            CouponAvailableGroupDto groupDto = new CouponAvailableGroupDto();
            BeanUtils.copyProperties(prodParam, groupDto);
            groupDto.setTransportId(transportId);
            // 根据商品ID与线路ID获取商品运费与清关费的单价
            this.coverPreferentialCondition(groupDto, prodParam);

            if (Objects.nonNull(prodParam.getActivityDtoList()) && CollectionUtil.isNotEmpty(prodParam.getActivityDtoList())) {
                for (CouponAvailableDto couponAvailableDto : prodParam.getActivityDtoList()) {
                    Map<String, BigDecimal> couponTotalChargeMap = couponConditionMap.get(couponAvailableDto.getCouponId());
                    if (Objects.nonNull(couponTotalChargeMap)) {
                        groupDto.setCouponTotalChargeVolume(couponTotalChargeMap.get("couponTotalChargeVolume"));
                        groupDto.setCouponTotalChargeWeight(couponTotalChargeMap.get("couponTotalChargeWeight"));
                        groupDto.setCouponTotalChargeQuantity(couponTotalChargeMap.get("couponTotalChargeQuantity"));
                        groupDto.setCouponTotalVolume(couponTotalChargeMap.get("couponTotalVolume"));
                        groupDto.setCouponTotalWeight(couponTotalChargeMap.get("couponTotalWeight"));
                        groupDto.setCouponTotalQuantity(couponTotalChargeMap.get("couponTotalQuantity"));
                    }
                    // 获取优惠活动使用条件
                    List<DiscountDetailedVO> detailedVOList = couponAvailableDto.getDiscountDetailedVOList();
                    // 类型 1:优惠卷 2:金额-满减 3:方数-满减 4 折扣 5 优惠 6 特价 7 区间价
                    this.availableActivityCouponAdd(couponAvailableDto, unitDOMap, detailedVOList, groupDto);
                }
            }
            if (CollectionUtil.isNotEmpty(groupDto.getCouponAvailableDtoList())) {
                List<CouponAvailableDto> couponAvailableDtoList = this.getMaxReduceAmount(groupDto.getCouponAvailableDtoList(), rmbCurrency);
                if (Objects.nonNull(couponAvailableDtoList) && CollectionUtil.isNotEmpty(couponAvailableDtoList)) {
                    groupDto.setCouponAvailableDtoList(couponAvailableDtoList);
                }
            }
            prodParam.setGroupDto(groupDto);
        }
    }

    @Override
    public CouponAvailableGroupDto getCouponUserAvailableDto(Integer orderType, Integer customsType, Integer isControl, Long userId,
                                                             Long consignorCustomerContactsId, Long consigneeCustomerContactsId,
                                                             Long lineId, Long channelId, ProdConditionParam prodParam, Map<Integer, UnitDO> unitDOMap,
                                                             Integer rmbCurrency) {
        CouponAvailableGroupDto groupDto = new CouponAvailableGroupDto();
        BeanUtils.copyProperties(prodParam, groupDto);

        // 根据商品ID与线路ID获取商品运费与清关费的单价
        this.coverPreferentialCondition(lineId, channelId, groupDto, prodParam);

        // 优惠活动列表
        List<CouponAvailableDto> activityDtoList = couponMapper.getActivityCouponAvailableDto(orderType,
                customsType, isControl, userId, consignorCustomerContactsId, consigneeCustomerContactsId,
                Collections.singletonList(prodParam), lineId, channelId, DateUtil.beginOfDay(new Date()));
        for (CouponAvailableDto couponAvailableDto : activityDtoList) {
            // 获取优惠活动使用条件
            List<DiscountDetailedVO> detailedVOList = couponAvailableDto.getDiscountDetailedVOList();
            if (CollectionUtil.isEmpty(detailedVOList)) {
                // 活动优惠条件清单为空，则忽略此优惠
                continue;
            }
            List<ProductDO> productDOList = couponAvailableDto.getProdList();
            if (CollectionUtil.isNotEmpty(productDOList)) {
                // 有指定商品条件
                long count = productDOList.stream().filter(productDO -> Objects.equals(productDO.getId(), groupDto.getProdId())).count();
                if (count > 0) {
                    if (Arrays.stream((couponAvailableDto.getBrand().split(StrUtil.COMMA))).map(Integer::valueOf).collect(Collectors.toList()).contains(groupDto.getBrandType())) {
                        // 类型 1:优惠卷 2:金额-满减 3:方数-满减 4 折扣 5 优惠 6 特价 7 区间价
                        this.availableActivityCouponAdd(couponAvailableDto, unitDOMap, detailedVOList, groupDto);
                    }
                }


            } else {
                // 类型 1:优惠卷 2:金额-满减 3:方数-满减 4 折扣 5 优惠 6 特价 7 区间价
                if (Arrays.stream((couponAvailableDto.getBrand().split(StrUtil.COMMA))).map(Integer::valueOf).collect(Collectors.toList()).contains(groupDto.getBrandType())) {
                    this.availableActivityCouponAdd(couponAvailableDto, unitDOMap, detailedVOList, groupDto);
                }

            }
        }
        if (CollectionUtil.isNotEmpty(groupDto.getCouponAvailableDtoList())) {
            List<CouponAvailableDto> couponAvailableDtoList = this.getMaxReduceAmount(groupDto.getCouponAvailableDtoList(), rmbCurrency);
            if (Objects.nonNull(couponAvailableDtoList) && CollectionUtil.isNotEmpty(couponAvailableDtoList)) {
                BigDecimal reduceAmount = BigDecimal.ZERO;
                BigDecimal clearanceReduceAmount = BigDecimal.ZERO;
                for (CouponAvailableDto availableDto : couponAvailableDtoList) {
                    reduceAmount = reduceAmount.add(Objects.nonNull(availableDto.getReduceAmount()) ? availableDto.getReduceAmount() : BigDecimal.ZERO);
                    clearanceReduceAmount = clearanceReduceAmount.add(Objects.nonNull(availableDto.getClearanceReduceAmount()) ? availableDto.getClearanceReduceAmount() : BigDecimal.ZERO);
                }
                prodParam.setDiscountFreightPrice(reduceAmount);
                // 相同货币单位的清关费优惠才能使用
                prodParam.setDiscountClearanceFeePrice(reduceAmount);
                groupDto.setCouponAvailableDtoList(couponAvailableDtoList);
                return groupDto;
            }
        }
        return null;
    }

    private void coverPreferentialCondition(Long lineId, Long channelId, CouponAvailableGroupDto groupDto, ProdConditionParam prodParam) {
        if (Objects.isNull(prodParam.getOneFreight()) || prodParam.getOneFreight().compareTo(BigDecimal.ZERO) == 0) {
//            ProductPriceDO productPriceDO = productPriceService.getOne(new LambdaQueryWrapper<ProductPriceDO>()
//                    .eq(ProductPriceDO::getProductId, prodParam.getProdId())
//                    .eq(ProductPriceDO::getWarehouseLineId, lineId)
//                    .and(we -> we.le(ProductPriceDO::getValidateStartDate, DateUtil.date())
//                            .or()
//                            .isNull(ProductPriceDO::getValidateStartDate))
//                    .and(we -> we.ge(ProductPriceDO::getValidateEndDate, DateUtil.date())
//                            .or()
//                            .isNull(ProductPriceDO::getValidateEndDate))
//                    .eq(null != channelId && 0 != channelId, ProductPriceDO::getShippingChannelId, channelId)
//                    .eq(ProductPriceDO::getDeleted, false)
//                    .last("limit 1"));
//            if (Objects.nonNull(productPriceDO)) {
//                if (productPriceDO.getPriceType() == 0) {
//                    groupDto.setClearanceFee(productPriceDO.getClearancePrice());
//                    groupDto.setClearanceFeeVolume(productPriceDO.getClearanceVolumeUnit());
//                    groupDto.setClearanceFeeCurrency(Math.toIntExact(productPriceDO.getClearancePriceUnit()));
//
//                    groupDto.setOrgFreightPrice(productPriceDO.getTransportPrice());
//                    groupDto.setFreight(productPriceDO.getTransportPrice());
//                    groupDto.setFreightVolume(productPriceDO.getTransportVolumeUnit());
//                    groupDto.setFreightCurrency(Math.toIntExact(productPriceDO.getTransportPriceUnit()));
//                } else {
//                    groupDto.setOrgFreightPrice(productPriceDO.getAllPrice());
//                    groupDto.setFreight(productPriceDO.getAllPrice());
//                    groupDto.setFreightVolume(productPriceDO.getAllVolumeUnit());
//                    groupDto.setFreightCurrency(Math.toIntExact(productPriceDO.getAllPriceUnit()));
//                }
//            }
        } else {
            groupDto.setOrgClearanceFeePrice(prodParam.getOrgClearanceFeePrice());
            groupDto.setClearanceFee(prodParam.getOneClearanceFee());
            groupDto.setClearanceFeeVolume(prodParam.getClearanceFeeVolume());
            groupDto.setClearanceFeeCurrency(prodParam.getClearanceFeeCurrency());

            groupDto.setOrgFreightPrice(prodParam.getOrgFreightPrice());
            groupDto.setFreight(prodParam.getOneFreight());
            groupDto.setFreightVolume(prodParam.getFreightVolume());
            groupDto.setFreightCurrency(prodParam.getFreightCurrency());
            if (prodParam.getBrandFreightPrice().compareTo(BigDecimal.ZERO) > 0 || prodParam.getBrandClearanceFeePrice().compareTo(BigDecimal.ZERO) > 0
                    || prodParam.getElectrifiedFreightPrice().compareTo(BigDecimal.ZERO) > 0 || prodParam.getElectrifiedClearanceFeePrice().compareTo(BigDecimal.ZERO) > 0
                    || prodParam.getLiquidFreightPrice().compareTo(BigDecimal.ZERO) > 0 || prodParam.getLiquidClearanceFeePrice().compareTo(BigDecimal.ZERO) > 0) {
                // 当有牌、带电、液体的加价大于0，则有特殊加价
                groupDto.setSpecialMarkup(true);
            }
        }
    }


    private void coverPreferentialCondition(CouponAvailableGroupDto groupDto, ProdConditionParam prodParam) {
//        if (Objects.nonNull(prodParam.getOneFreight()) && prodParam.getOneFreight().compareTo(BigDecimal.ZERO) > 0) {
            groupDto.setOrgClearanceFeePrice(prodParam.getOrgClearanceFeePrice());
            groupDto.setClearanceFee(Objects.isNull(prodParam.getOneClearanceFee()) ? BigDecimal.ZERO : prodParam.getOneClearanceFee());
            groupDto.setClearanceFeeVolume(prodParam.getClearanceFeeVolume());
            groupDto.setClearanceFeeCurrency(prodParam.getClearanceFeeCurrency());

            groupDto.setOrgFreightPrice(prodParam.getOrgFreightPrice());
            // 此处是在将所有的费用计算后的成交单价，需要排除佣金来匹配优惠
            groupDto.setFreight(Objects.isNull(prodParam.getOneFreight()) ?
                    BigDecimal.ZERO : (prodParam.getOneFreight().subtract(Objects.nonNull(prodParam.getPlainCommission()) ? prodParam.getPlainCommission() : BigDecimal.ZERO)));
            groupDto.setFreightVolume(prodParam.getFreightVolume());
            groupDto.setFreightCurrency(prodParam.getFreightCurrency());
            if (prodParam.getBrandFreightPrice().compareTo(BigDecimal.ZERO) > 0 || prodParam.getBrandClearanceFeePrice().compareTo(BigDecimal.ZERO) > 0
                    || prodParam.getElectrifiedFreightPrice().compareTo(BigDecimal.ZERO) > 0 || prodParam.getElectrifiedClearanceFeePrice().compareTo(BigDecimal.ZERO) > 0
                    || prodParam.getLiquidFreightPrice().compareTo(BigDecimal.ZERO) > 0 || prodParam.getLiquidClearanceFeePrice().compareTo(BigDecimal.ZERO) > 0) {
                // 当有牌、带电、液体的加价大于0，则有特殊加价
                groupDto.setSpecialMarkup(true);
            }
//        }
    }

    private void coverPreferentialConditions(Long lineId, Long channelId, List<CouponAvailableGroupDto> groupDtoList, ProdConditionParam prodParam) {
        if (Objects.isNull(prodParam.getOneSeaFreight()) || prodParam.getOneFreight().compareTo(BigDecimal.ZERO) == 0) {
            ProductPriceDO productPriceDO = productPriceService.getOne(new LambdaQueryWrapper<ProductPriceDO>()
                    .eq(ProductPriceDO::getProductId, prodParam.getProdId())
                    .eq(ProductPriceDO::getWarehouseLineId, lineId)
                    .and(we -> we.le(ProductPriceDO::getValidateStartDate, DateUtil.date())
                            .or()
                            .isNull(ProductPriceDO::getValidateStartDate))
                    .and(we -> we.ge(ProductPriceDO::getValidateEndDate, DateUtil.date())
                            .or()
                            .isNull(ProductPriceDO::getValidateEndDate))
                    .eq(null != channelId && 0 != channelId, ProductPriceDO::getShippingChannelId, channelId)
                    .eq(ProductPriceDO::getDeleted, false)
                    .last("limit 1"));
            if (Objects.nonNull(productPriceDO)) {
                CouponAvailableGroupDto groupDto = new CouponAvailableGroupDto();
                BeanUtils.copyProperties(prodParam, groupDto);
                if (productPriceDO.getPriceType() == 0) {
                    groupDto.setClearanceFee(productPriceDO.getClearancePrice());
                    groupDto.setClearanceFeeVolume(productPriceDO.getClearanceVolumeUnit());
                    groupDto.setClearanceFeeCurrency(Math.toIntExact(productPriceDO.getClearancePriceUnit()));

                    groupDto.setFreight(productPriceDO.getAllPrice());
                    groupDto.setFreightVolume(productPriceDO.getAllVolumeUnit());
                    groupDto.setFreightCurrency(Math.toIntExact(productPriceDO.getAllPriceUnit()));
                } else {
                    groupDto.setFreight(productPriceDO.getTransportPrice());
                    groupDto.setFreightVolume(productPriceDO.getTransportVolumeUnit());
                    groupDto.setFreightCurrency(Math.toIntExact(productPriceDO.getTransportPriceUnit()));
                }
                groupDtoList.add(groupDto);
            }
        } else {
            CouponAvailableGroupDto groupDto = new CouponAvailableGroupDto();
            BeanUtils.copyProperties(prodParam, groupDto);
            groupDto.setClearanceFee(prodParam.getOneClearanceFee());
            groupDto.setClearanceFeeVolume(prodParam.getClearanceFeeVolume());
            groupDto.setClearanceFeeCurrency(prodParam.getClearanceFeeCurrency());

            groupDto.setFreight(prodParam.getOneFreight());
            groupDto.setFreightVolume(prodParam.getFreightVolume());
            groupDto.setFreightCurrency(prodParam.getFreightCurrency());
            groupDtoList.add(groupDto);
        }
    }

    /**
     * 添加用户可用优惠活动
     *
     * @param couponAvailableDto 用户的优惠活动详情
     * @param unitDOMap          在用计量单位
     * @param detailedVOList     优惠活动中的条件清单
     * @param groupDto           商品费用清单列表
     * @return 开关属性值
     */
    private void availableActivityCouponAdd(CouponAvailableDto couponAvailableDto, Map<Integer, UnitDO> unitDOMap,
                                            List<DiscountDetailedVO> detailedVOList, CouponAvailableGroupDto groupDto) {
        List<CouponAvailableDto> dtoList = groupDto.getCouponAvailableDtoList();
        boolean isAir = groupDto.getTransportId() != 1 && groupDto.getTransportId() != 2;
        BigDecimal volume = groupDto.getCouponTotalVolume();
        BigDecimal weight = groupDto.getCouponTotalWeight();
        BigDecimal quantity = groupDto.getCouponTotalQuantity();
        BigDecimal chargeVolume = groupDto.getCouponTotalChargeVolume();
        BigDecimal chargeWeight = groupDto.getCouponTotalChargeWeight();
        BigDecimal chargeQuantity = groupDto.getCouponTotalChargeQuantity();
        // TODO 返回满足使用优惠劵的条件一条数据到列表数据中
        switch (couponAvailableDto.getType()) {
            case 2:
                List<DiscountDetailedVO> fullReduceList = new ArrayList<>();
                List<DiscountDetailedVO> amountFullReduceList = detailedVOList.stream().filter(vo -> null != vo.getFullCurrencyId()).collect(Collectors.toList());
                // 金额-满减
                List<DiscountDetailedVO> amountFullReduceVOList = fullDiscount(couponAvailableDto, amountFullReduceList, groupDto);
                if (CollectionUtil.isNotEmpty(amountFullReduceVOList)) {
                    fullReduceList.addAll(amountFullReduceVOList);
                }
                // 方数/重量/个数-满减
                List<DiscountDetailedVO> unitFullReduceList = detailedVOList.stream()
                        .filter(vo -> Objects.nonNull(vo.getProdUnit()) && Objects.nonNull(unitDOMap.get(vo.getProdUnit().intValue())) &&
                                ((StringUtils.equals("m³", unitDOMap.get(vo.getProdUnit().intValue()).getFuhao()) && vo.getFullAmount().compareTo(!isAir && vo.getType() == 1 ? chargeVolume : volume) <= 0) ||
                                        (StringUtils.equals("kg", unitDOMap.get(vo.getProdUnit().intValue()).getFuhao()) && vo.getFullAmount().compareTo(!isAir && vo.getType() == 1 ? chargeWeight : weight) <= 0) ||
                                        (StringUtils.equals("a", unitDOMap.get(vo.getProdUnit().intValue()).getFuhao()) && vo.getFullAmount().compareTo(!isAir && vo.getType() == 1 ? chargeQuantity : quantity) <= 0)))
                        .collect(Collectors.toList());
                if (CollectionUtil.isNotEmpty(unitFullReduceList)) {
                    fullReduceList.addAll(unitFullReduceList);
                }
                DiscountDetailedVO feeDetailedVO = fullReduceList.stream().filter(vo -> vo.getType() == 1 && Objects.equals(vo.getReduceCurrencyId(), groupDto.getFreightCurrency())).max(Comparator.comparing(DiscountDetailedVO::getReduceAmount)).orElse(null);
                if (Objects.nonNull(feeDetailedVO)) {
                    couponAvailableDto.setReduceAmount(feeDetailedVO.getReduceAmount());
                    couponAvailableDto.setReduceCurrencyId(feeDetailedVO.getReduceCurrencyId());
                    couponAvailableDto.setFreightDetailed(feeDetailedVO);
                }
                DiscountDetailedVO clearanceDetailedVO = fullReduceList.stream().filter(vo -> vo.getType() == 2 && Objects.equals(vo.getReduceCurrencyId(), groupDto.getClearanceFeeCurrency())).max(Comparator.comparing(DiscountDetailedVO::getReduceAmount)).orElse(null);
                if (Objects.nonNull(clearanceDetailedVO) && groupDto.getClearanceFee().compareTo(BigDecimal.ZERO) > 0) {
                    couponAvailableDto.setClearanceReduceAmount(clearanceDetailedVO.getReduceAmount());
                    couponAvailableDto.setClearanceReduceCurrencyId(clearanceDetailedVO.getReduceCurrencyId());
                    couponAvailableDto.setClearanceDetailed(clearanceDetailedVO);
                }
                if (Objects.nonNull(feeDetailedVO) || Objects.nonNull(clearanceDetailedVO)) {
                    dtoList.add(couponAvailableDto);
                }
                break;
            case 4:
                // 折扣
                List<DiscountDetailedVO> discountDetailedVOList = new ArrayList<>();
                List<DiscountDetailedVO> amountDiscountList = detailedVOList.stream().filter(vo -> null != vo.getFullCurrencyId()).collect(Collectors.toList());
                List<DiscountDetailedVO> amountDiscountVOList = fullDiscount(couponAvailableDto, amountDiscountList, groupDto);
                if (CollectionUtil.isNotEmpty(amountDiscountVOList)) {
                    discountDetailedVOList.addAll(amountDiscountVOList);
                }
                // 方数/重量/个数-满收
                List<DiscountDetailedVO> unitDiscountVOList = detailedVOList.stream()
                        .filter(vo -> Objects.nonNull(vo.getProdUnit()) && Objects.nonNull(unitDOMap.get(vo.getProdUnit().intValue())) &&
                                ((StringUtils.equals("m³", unitDOMap.get(vo.getProdUnit().intValue()).getFuhao()) && vo.getFullAmount().compareTo(!isAir && vo.getType() == 1 ? chargeVolume : volume) <= 0) ||
                                        (StringUtils.equals("kg", unitDOMap.get(vo.getProdUnit().intValue()).getFuhao()) && vo.getFullAmount().compareTo(!isAir && vo.getType() == 1 ? chargeWeight : weight) <= 0) ||
                                        (StringUtils.equals("a", unitDOMap.get(vo.getProdUnit().intValue()).getFuhao()) && vo.getFullAmount().compareTo(!isAir && vo.getType() == 1 ? chargeQuantity : quantity) <= 0)))
                        .collect(Collectors.toList());
                if (CollectionUtil.isNotEmpty(unitDiscountVOList)) {
                    discountDetailedVOList.addAll(unitDiscountVOList);
                }
                if (CollectionUtil.isNotEmpty(discountDetailedVOList)) {
                    for (DiscountDetailedVO discountDetailedVO : discountDetailedVOList) {
                        BigDecimal discountAmount = discountDetailedVO.getType() == 1 ?
                                groupDto.getFreight().subtract((discountDetailedVO.getReduceAmount().divide(new BigDecimal("100"), 2, RoundingMode.HALF_UP)).multiply(groupDto.getFreight())) :
                                groupDto.getClearanceFee().subtract((discountDetailedVO.getReduceAmount().divide(new BigDecimal("100"), 2, RoundingMode.HALF_UP)).multiply(groupDto.getClearanceFee()));
                        if (discountDetailedVO.getType() == 1) {
                            couponAvailableDto.setReduceAmount(discountAmount);
                            couponAvailableDto.setReduceCurrencyId(groupDto.getFreightCurrency());
                            couponAvailableDto.setFreightDetailed(discountDetailedVO);
                        } else {
                            if (groupDto.getClearanceFee().compareTo(BigDecimal.ZERO) > 0) {
                                couponAvailableDto.setClearanceReduceAmount(discountAmount);
                                couponAvailableDto.setClearanceReduceCurrencyId(groupDto.getClearanceFeeCurrency());
                                couponAvailableDto.setClearanceDetailed(discountDetailedVO);
                            }
                        }
                    }
                    dtoList.add(couponAvailableDto);
                }
                break;
            case 5:
                // 优惠
                List<DiscountDetailedVO> couponDetailedVOList = new ArrayList<>();
                List<DiscountDetailedVO> amountFullCouponVOList = detailedVOList.stream().filter(vo -> null != vo.getFullCurrencyId()).collect(Collectors.toList());
                // 金额-满收
                List<DiscountDetailedVO> amountFullCouponList = fullDiscount(couponAvailableDto, amountFullCouponVOList, groupDto);
                if (CollectionUtil.isNotEmpty(amountFullCouponList)) {
                    couponDetailedVOList.addAll(amountFullCouponList);
                }
                // 方数/重量/个数-满收
                List<DiscountDetailedVO> unitFullCouponList = detailedVOList.stream()
                        .filter(vo -> Objects.nonNull(vo.getProdUnit()) && Objects.nonNull(unitDOMap.get(vo.getProdUnit().intValue())) &&
                                ((StringUtils.equals("m³", unitDOMap.get(vo.getProdUnit().intValue()).getFuhao()) && vo.getFullAmount().compareTo(!isAir && vo.getType() == 1 ? chargeVolume : volume) <= 0) ||
                                        (StringUtils.equals("kg", unitDOMap.get(vo.getProdUnit().intValue()).getFuhao()) && vo.getFullAmount().compareTo(!isAir && vo.getType() == 1 ? chargeWeight : weight) <= 0) ||
                                        (StringUtils.equals("a", unitDOMap.get(vo.getProdUnit().intValue()).getFuhao()) && vo.getFullAmount().compareTo(!isAir && vo.getType() == 1 ? chargeQuantity : quantity) <= 0)))
                        .collect(Collectors.toList());
                if (CollectionUtil.isNotEmpty(unitFullCouponList)) {
                    couponDetailedVOList.addAll(unitFullCouponList);
                }
                if (CollectionUtil.isNotEmpty(couponDetailedVOList)) {
                    for (DiscountDetailedVO couponDetailedVO : couponDetailedVOList) {
                        if (couponDetailedVO.getType() == 1 && Objects.equals(couponDetailedVO.getNetReceiptsCurrencyId(), groupDto.getFreightCurrency())) {
                            // 满 x  收 y TODO 是否复数乘积优惠
                            couponAvailableDto.setReduceAmount(groupDto.getFreight().subtract(couponDetailedVO.getNetReceiptsAmount()));
                            couponAvailableDto.setReduceCurrencyId(couponDetailedVO.getNetReceiptsCurrencyId());
                            couponAvailableDto.setFreightDetailed(couponDetailedVO);
                        } else {
                            if (Objects.equals(couponDetailedVO.getNetReceiptsCurrencyId(), groupDto.getClearanceFeeCurrency()) && groupDto.getClearanceFee().compareTo(BigDecimal.ZERO) > 0) {
                                couponAvailableDto.setClearanceReduceAmount(groupDto.getClearanceFee().subtract(couponDetailedVO.getNetReceiptsAmount()));
                                couponAvailableDto.setClearanceReduceCurrencyId(couponDetailedVO.getNetReceiptsCurrencyId());
                                couponAvailableDto.setClearanceDetailed(couponDetailedVO);
                            }
                        }

                    }
                    dtoList.add(couponAvailableDto);
                }
                break;
            case 6:
                if (groupDto.isSpecialMarkup()) {
                    break;
                } else {
                    // 特价
                    DiscountDetailedVO feeSpecialDetailedVO = detailedVOList.stream()
                            .filter(vo -> vo.getType() == 1 && Objects.nonNull(groupDto.getFreightVolume()) &&
                                    Objects.equals(groupDto.getFreightVolume(), vo.getProdUnit()) &&
                                    Objects.nonNull(groupDto.getFreightCurrency()) &&
                                    Objects.equals(groupDto.getFreightCurrency(), vo.getNetReceiptsCurrencyId()))
                            .min(Comparator.comparing(DiscountDetailedVO::getNetReceiptsAmount)).orElse(null);
                    if (Objects.nonNull(feeSpecialDetailedVO) && groupDto.getOrgFreightPrice().compareTo(BigDecimal.ZERO) > 0) {
                        couponAvailableDto.setReduceAmount(groupDto.getOrgFreightPrice().subtract(feeSpecialDetailedVO.getNetReceiptsAmount()));
                        couponAvailableDto.setReduceCurrencyId(feeSpecialDetailedVO.getNetReceiptsCurrencyId());
                        couponAvailableDto.setFreightDetailed(feeSpecialDetailedVO);
                    }
                    DiscountDetailedVO clearanceSpecialDetailedVO = detailedVOList.stream().filter(vo ->
                                    vo.getType() == 2 && Objects.nonNull(groupDto.getClearanceFeeVolume()) &&
                                            Objects.equals(groupDto.getClearanceFeeVolume(), vo.getProdUnit()) &&
                                            Objects.nonNull(groupDto.getClearanceFeeCurrency()) &&
                                            Objects.equals(groupDto.getClearanceFeeCurrency(), vo.getNetReceiptsCurrencyId()))
                            .min(Comparator.comparing(DiscountDetailedVO::getNetReceiptsAmount)).orElse(null);
                    if (Objects.nonNull(clearanceSpecialDetailedVO) && groupDto.getOrgClearanceFeePrice().compareTo(BigDecimal.ZERO) > 0) {
                        couponAvailableDto.setClearanceReduceAmount(groupDto.getOrgClearanceFeePrice().subtract(clearanceSpecialDetailedVO.getNetReceiptsAmount()));
                        couponAvailableDto.setClearanceReduceCurrencyId(clearanceSpecialDetailedVO.getNetReceiptsCurrencyId());
                        couponAvailableDto.setClearanceDetailed(clearanceSpecialDetailedVO);
                    }
                    if (Objects.nonNull(feeSpecialDetailedVO) || Objects.nonNull(clearanceSpecialDetailedVO)) {
                        dtoList.add(couponAvailableDto);
                    }
                    break;
                }
            case 7:
                // 区间价
                DiscountDetailedVO feeSectionDetailedVO = detailedVOList.stream().filter(vo -> vo.getType() == 1 &&
                        ((Objects.nonNull(vo.getStartAndEndCurrencyId()) && Objects.equals(groupDto.getFreightCurrency(), vo.getStartAndEndCurrencyId()) &&
                                (vo.getStartAmount().compareTo(groupDto.getFreight()) < 0 &&
                                        (vo.getEndAmount() == null ||
                                                vo.getEndAmount().compareTo(BigDecimal.ZERO) == 0 ||
                                                vo.getEndAmount().compareTo(groupDto.getFreight()) >= 0)
                                )) || (Objects.nonNull(vo.getProdUnit()) && Objects.nonNull(unitDOMap.get(vo.getProdUnit().intValue())) &&
                                (((StringUtils.equals("m³", unitDOMap.get(vo.getProdUnit().intValue()).getFuhao()) && vo.getFullAmount().compareTo(!isAir && vo.getType() == 1 ? chargeVolume : volume) < 0) && (vo.getEndAmount() == null ||
                                        vo.getEndAmount().compareTo(BigDecimal.ZERO) == 0 ||
                                        vo.getEndAmount().compareTo(!isAir && vo.getType() == 1 ? chargeVolume : volume) >= 0))) ||
                                (((StringUtils.equals("kg", unitDOMap.get(vo.getProdUnit().intValue()).getFuhao()) && vo.getFullAmount().compareTo(!isAir && vo.getType() == 1 ? chargeWeight : weight) < 0) && (vo.getEndAmount() == null ||
                                        vo.getEndAmount().compareTo(BigDecimal.ZERO) == 0 ||
                                        vo.getEndAmount().compareTo(!isAir && vo.getType() == 1 ? chargeWeight : weight) >= 0))) ||
                                (((StringUtils.equals("a", unitDOMap.get(vo.getProdUnit().intValue()).getFuhao()) && vo.getFullAmount().compareTo(!isAir && vo.getType() == 1 ? chargeQuantity : quantity) < 0) && (vo.getEndAmount() == null ||
                                        vo.getEndAmount().compareTo(BigDecimal.ZERO) == 0 ||
                                        vo.getEndAmount().compareTo(!isAir && vo.getType() == 1 ? chargeQuantity : quantity) >= 0))))
                        )).min(Comparator.comparing(DiscountDetailedVO::getNetReceiptsAmount)).orElse(null);
                if (Objects.nonNull(feeSectionDetailedVO) && Objects.equals(feeSectionDetailedVO.getNetReceiptsCurrencyId(), groupDto.getFreightCurrency())) {
                    couponAvailableDto.setReduceAmount(groupDto.getFreight().subtract(feeSectionDetailedVO.getNetReceiptsAmount()));
                    couponAvailableDto.setReduceCurrencyId(feeSectionDetailedVO.getNetReceiptsCurrencyId());
                    couponAvailableDto.setFreightDetailed(feeSectionDetailedVO);
                }
                DiscountDetailedVO clearanceSectionDetailedVO = detailedVOList.stream().filter(vo -> vo.getType() == 2 &&
                        ((Objects.nonNull(vo.getStartAndEndCurrencyId()) && Objects.equals(groupDto.getClearanceFeeCurrency(), vo.getStartAndEndCurrencyId()) &&
                                (vo.getStartAmount().compareTo(groupDto.getClearanceFee()) < 0 &&
                                        (vo.getEndAmount() == null ||
                                                vo.getEndAmount().compareTo(BigDecimal.ZERO) == 0 ||
                                                vo.getEndAmount().compareTo(groupDto.getClearanceFee()) >= 0))) ||
                                (Objects.nonNull(vo.getProdUnit()) && Objects.nonNull(unitDOMap.get(vo.getProdUnit().intValue())) &&
                                        (((StringUtils.equals("m³", unitDOMap.get(vo.getProdUnit().intValue()).getFuhao()) && vo.getFullAmount().compareTo(!isAir && vo.getType() == 1 ? chargeVolume : volume) < 0) && (vo.getEndAmount() == null ||
                                                vo.getEndAmount().compareTo(BigDecimal.ZERO) == 0 ||
                                                vo.getEndAmount().compareTo(!isAir && vo.getType() == 1 ? chargeVolume : volume) >= 0))) ||
                                        (((StringUtils.equals("kg", unitDOMap.get(vo.getProdUnit().intValue()).getFuhao()) && vo.getFullAmount().compareTo(!isAir && vo.getType() == 1 ? chargeWeight : weight) < 0) && (vo.getEndAmount() == null ||
                                                vo.getEndAmount().compareTo(BigDecimal.ZERO) == 0 ||
                                                vo.getEndAmount().compareTo(!isAir && vo.getType() == 1 ? chargeWeight : weight) >= 0))) ||
                                        (((StringUtils.equals("a", unitDOMap.get(vo.getProdUnit().intValue()).getFuhao()) && vo.getFullAmount().compareTo(!isAir && vo.getType() == 1 ? chargeQuantity : quantity) < 0) && (vo.getEndAmount() == null ||
                                                vo.getEndAmount().compareTo(BigDecimal.ZERO) == 0 ||
                                                vo.getEndAmount().compareTo(!isAir && vo.getType() == 1 ? chargeQuantity : quantity) >= 0))))
                        )).min(Comparator.comparing(DiscountDetailedVO::getNetReceiptsAmount)).orElse(null);
                if (Objects.nonNull(clearanceSectionDetailedVO) && Objects.equals(clearanceSectionDetailedVO.getNetReceiptsCurrencyId(), groupDto.getClearanceFeeCurrency()) && groupDto.getClearanceFee().compareTo(BigDecimal.ZERO) > 0) {
                    couponAvailableDto.setClearanceReduceAmount(groupDto.getClearanceFee().subtract(clearanceSectionDetailedVO.getNetReceiptsAmount()));
                    couponAvailableDto.setClearanceReduceCurrencyId(clearanceSectionDetailedVO.getNetReceiptsCurrencyId());
                    couponAvailableDto.setClearanceDetailed(clearanceSectionDetailedVO);
                }
                if (Objects.nonNull(feeSectionDetailedVO) || Objects.nonNull(clearanceSectionDetailedVO)) {
                    dtoList.add(couponAvailableDto);
                }
                break;
            default:
                break;

        }
        groupDto.setCouponAvailableDtoList(dtoList);
    }

    /**
     * 计算优惠满足(金额)条件
     *
     * @param couponAvailableDto
     * @param detailedVOList
     * @param groupDto
     * @return
     */
    private List<DiscountDetailedVO> fullDiscount(CouponAvailableDto couponAvailableDto, List<DiscountDetailedVO> detailedVOList,
                                                  CouponAvailableGroupDto groupDto) {
        List<DiscountDetailedVO> detailedVOS = detailedVOList.stream().filter(discountDetailedVO ->
                ((discountDetailedVO.getType() == 1 && Objects.equals(groupDto.getFreightCurrency(), discountDetailedVO.getFullCurrencyId()) &&
                        discountDetailedVO.getFullAmount().compareTo(groupDto.getFreight()) <= 0) ||
                        (discountDetailedVO.getType() == 2 && Objects.equals(groupDto.getClearanceFeeCurrency(), discountDetailedVO.getFullCurrencyId()) &&
                                discountDetailedVO.getFullAmount().compareTo(groupDto.getClearanceFee()) <= 0)
                )).collect(Collectors.toList());
        List<DiscountDetailedVO> discountDetailedVOList = new ArrayList<>();
        if (couponAvailableDto.getType() == 2 || couponAvailableDto.getType() == 5) {
            // 运费
            DiscountDetailedVO feeDetailedVO = detailedVOS.stream()
                    .filter(discountDetailedVO -> Objects.nonNull(discountDetailedVO.getReduceAmount()) && discountDetailedVO.getType() == 1)
                    .max(Comparator.comparing(DiscountDetailedVO::getReduceAmount)).orElse(null);
            if (Objects.nonNull(feeDetailedVO)) {
                discountDetailedVOList.add(feeDetailedVO);
            }
            // 清关费
            DiscountDetailedVO clearanceDetailedVO = detailedVOS.stream()
                    .filter(discountDetailedVO -> Objects.nonNull(discountDetailedVO.getReduceAmount()) && discountDetailedVO.getType() == 2)
                    .max(Comparator.comparing(DiscountDetailedVO::getReduceAmount)).orElse(null);
            if (Objects.nonNull(clearanceDetailedVO)) {
                discountDetailedVOList.add(clearanceDetailedVO);
            }
        }
        if (couponAvailableDto.getType() == 4) {
            // 运费
            DiscountDetailedVO feeDetailedVO = detailedVOS.stream()
                    .filter(discountDetailedVO -> Objects.nonNull(discountDetailedVO.getReduceAmount()) && discountDetailedVO.getType() == 1)
                    .min(Comparator.comparing(DiscountDetailedVO::getReduceAmount)).orElse(null);
            if (Objects.nonNull(feeDetailedVO)) {
                discountDetailedVOList.add(feeDetailedVO);
            }
            // 清关费
            DiscountDetailedVO clearanceDetailedVO = detailedVOS.stream()
                    .filter(discountDetailedVO -> Objects.nonNull(discountDetailedVO.getReduceAmount()) && discountDetailedVO.getType() == 2)
                    .min(Comparator.comparing(DiscountDetailedVO::getReduceAmount)).orElse(null);
            if (Objects.nonNull(clearanceDetailedVO)) {
                discountDetailedVOList.add(clearanceDetailedVO);
            }
        }
        return discountDetailedVOList;
    }

    /**
     * 添加用户可用优惠劵列表
     *
     * @param costDto       费用清单
     * @param couponDtoList 优惠劵列表
     */
//    private List<CouponAvailableDto> availableCouponAdd(CostDto costDto, List<CouponAvailableDto> couponDtoList) {
//        List<CouponAvailableDto> dtoList = new ArrayList<>();
//        // 获取所有货币
//        List<CurrencyDO> currencyDOList = currencyService.list();
//        // 美元货币ID
//        Integer usdCurrency = currencyDOList.stream().filter(currencyDO -> StringUtils.equals("$", currencyDO.getFuhao())).findFirst().get().getId();
//
//        // 人民币货币ID
//        Integer rmbCurrency = currencyDOList.stream().filter(currencyDO -> StringUtils.equals("￥", currencyDO.getFuhao())).findFirst().get().getId();
//
//        // 奈拉货币ID
//        Integer ngnCurrency = currencyDOList.stream().filter(currencyDO -> StringUtils.equals("N", currencyDO.getFuhao())).findFirst().get().getId();
//
//        couponDtoList = couponDtoList.stream().sorted(Comparator.comparing(CouponAvailableDto::getReduceAmount).reversed()).collect(Collectors.toList());
//        List<FeeDto> feeDtoList = costDto.getFeeDtoList();
//        if (CollectionUtil.isEmpty(feeDtoList)) {
//            return dtoList;
//        }
//        // TODO 货币比较不能固定
//        BigDecimal feeNGN = feeDtoList.stream()
//                .filter(feeDto -> Objects.equals(ngnCurrency, feeDto.getCurrencyId())
//                        && !Objects.equals(feeDto.getFeeType(), FeeTypeEnum.DISCOUNT_FEE.getType())
//                        && !Objects.equals(feeDto.getFeeType(), FeeTypeEnum.WORTH.getType()))
//                .map(FeeDto::getAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
//        BigDecimal feeRMB = feeDtoList.stream()
//                .filter(feeDto -> Objects.equals(rmbCurrency, feeDto.getCurrencyId())
//                        && !Objects.equals(feeDto.getFeeType(), FeeTypeEnum.DISCOUNT_FEE.getType())
//                        && !Objects.equals(feeDto.getFeeType(), FeeTypeEnum.WORTH.getType()))
//                .map(FeeDto::getAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
//        BigDecimal feeUSD = feeDtoList.stream()
//                .filter(feeDto -> Objects.equals(usdCurrency, feeDto.getCurrencyId())
//                        && !Objects.equals(feeDto.getFeeType(), FeeTypeEnum.DISCOUNT_FEE.getType())
//                        && !Objects.equals(feeDto.getFeeType(), FeeTypeEnum.WORTH.getType()))
//                .map(FeeDto::getAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
//        List<CouponAvailableDto> availableDtoList = couponDtoList.stream().filter(dto -> dto.getUseStatus() != null && dto.getUseStatus() != 1).collect(Collectors.toList());
//        CouponAvailableDto usdCouponAvailableDto = availableDtoList.stream().filter(dto -> Objects.equals(usdCurrency, dto.getConditionCurrencyId()) &&
//                feeUSD.compareTo(BigDecimal.ZERO) > 0 && dto.getCashCondition().compareTo(feeUSD) <= 0).max(Comparator.comparing(CouponAvailableDto::getReduceAmount)).orElse(null);
//        if (Objects.nonNull(usdCouponAvailableDto)) {
//            dtoList.add(usdCouponAvailableDto);
//        }
//        CouponAvailableDto rmbCouponAvailableDto = availableDtoList.stream().filter(dto -> Objects.equals(rmbCurrency, dto.getConditionCurrencyId()) &&
//                feeUSD.compareTo(BigDecimal.ZERO) > 0 && dto.getCashCondition().compareTo(feeRMB) <= 0).max(Comparator.comparing(CouponAvailableDto::getReduceAmount)).orElse(null);
//        if (Objects.nonNull(rmbCouponAvailableDto)) {
//            dtoList.add(rmbCouponAvailableDto);
//        }
//        CouponAvailableDto ngnCouponAvailableDto = availableDtoList.stream().filter(dto -> Objects.equals(ngnCurrency, dto.getConditionCurrencyId()) &&
//                feeUSD.compareTo(BigDecimal.ZERO) > 0 && dto.getCashCondition().compareTo(feeNGN) <= 0).max(Comparator.comparing(CouponAvailableDto::getReduceAmount)).orElse(null);
//        if (Objects.nonNull(ngnCouponAvailableDto)) {
//            dtoList.add(ngnCouponAvailableDto);
//        }
//        if (CollectionUtil.isNotEmpty(dtoList)) {
//            return this.comparingReduceAmount(dtoList);
//        }
//        return dtoList;
//    }

    /**
     * 优惠排序
     *
     * @param dtoList
     * @return
     */
    private List<CouponAvailableDto> comparingReduceAmount(List<CouponAvailableDto> dtoList) {
        List<CouponAvailableDto> comparingDtoList = dtoList.stream().sorted(Comparator.comparing(CouponAvailableDto::getReduceAmount).reversed()).collect(Collectors.toList());
        comparingDtoList.get(0).setIsDefault(Boolean.TRUE);
        return comparingDtoList;
    }

    /**
     * 最大优惠
     *
     * @param dtoList     优惠列表
     * @param rmbCurrency rmb货币id
     * @return
     */
    private List<CouponAvailableDto> getMaxReduceAmount(List<CouponAvailableDto> dtoList, Integer rmbCurrency) {
        // 汇率换算运费和清关费的rmb总额度
        dtoList = dtoList.stream().peek(dto -> {
            // 计算rmb优惠总额
            ExchangeRateRespDTO freightRateRespDTO = new ExchangeRateRespDTO();
            if (Objects.nonNull(dto.getReduceCurrencyId())) {
                freightRateRespDTO = currencyApi.getCurrencyRate(dto.getReduceCurrencyId().longValue(), rmbCurrency.longValue());
                if (Objects.isNull(freightRateRespDTO)) {
                    // RMB兑RMB，1：1
                    freightRateRespDTO = new ExchangeRateRespDTO();
                    freightRateRespDTO.setSourceCurrencyId(dto.getReduceCurrencyId().longValue());
                    freightRateRespDTO.setTargetCurrencyId(rmbCurrency.longValue());
                    freightRateRespDTO.setCurrencyRate(new BigDecimal("1"));
                }
            }
            BigDecimal freightRMBReduceAmount = dto.getReduceAmount().compareTo(BigDecimal.ZERO) > 0 && Objects.nonNull(dto.getReduceCurrencyId()) &&
                    Objects.nonNull(freightRateRespDTO.getCurrencyRate()) && freightRateRespDTO.getCurrencyRate().compareTo(BigDecimal.ZERO) > 0 ?
                    dto.getReduceAmount().multiply(freightRateRespDTO.getCurrencyRate()).setScale(2, RoundingMode.HALF_UP) : BigDecimal.ZERO;
            ExchangeRateRespDTO clearanceFeeRateRespDTO = new ExchangeRateRespDTO();
            if (Objects.nonNull(dto.getClearanceReduceCurrencyId())) {
                clearanceFeeRateRespDTO = currencyApi.getCurrencyRate(dto.getClearanceReduceCurrencyId().longValue(), rmbCurrency.longValue());
                if (Objects.isNull(clearanceFeeRateRespDTO)) {
                    // RMB兑RMB，1：1
                    clearanceFeeRateRespDTO = new ExchangeRateRespDTO();
                    clearanceFeeRateRespDTO.setSourceCurrencyId(rmbCurrency.longValue());
                    clearanceFeeRateRespDTO.setTargetCurrencyId(rmbCurrency.longValue());
                    clearanceFeeRateRespDTO.setCurrencyRate(new BigDecimal("1"));
                }
            }
            BigDecimal clearanceFeeRMBReduceAmount = dto.getClearanceReduceAmount().compareTo(BigDecimal.ZERO) > 0 && Objects.nonNull(dto.getClearanceReduceCurrencyId()) &&
                    Objects.nonNull(clearanceFeeRateRespDTO.getCurrencyRate()) && clearanceFeeRateRespDTO.getCurrencyRate().compareTo(BigDecimal.ZERO) > 0 ?
                    dto.getClearanceReduceAmount().multiply(clearanceFeeRateRespDTO.getCurrencyRate()).setScale(2, RoundingMode.HALF_UP) : BigDecimal.ZERO;
            dto.setTotalRMBReduceAmount(freightRMBReduceAmount.add(clearanceFeeRMBReduceAmount));
            // 封装优惠详情
            CouponInfoVO infoVO = new CouponInfoVO();
            coverCouponInfoVO(dto, infoVO);
            dto.setCouponInfoVOList(Collections.singletonList(infoVO));
        }).collect(Collectors.toList());
        // TODO 处理可叠加优惠额度

        // 获得最大优惠额度
        List<CouponAvailableDto> comparingDtoList = dtoList.stream().sorted(Comparator.comparing(CouponAvailableDto::getTotalRMBReduceAmount).reversed()).collect(Collectors.toList());

        if (CollectionUtil.isNotEmpty(comparingDtoList) && comparingDtoList.size() > 0) {
            List<CouponAvailableDto> couponAvailableDtoList = new ArrayList<>();
            CouponAvailableDto dto = comparingDtoList.get(0);
            couponAvailableDtoList.add(dto);
            if (!dto.getIsSimilarSuperposition()) {
                // 可用叠加优惠
                CouponAvailableDto similarDto = comparingDtoList.stream().filter(couponAvailableDto -> !Objects.equals(dto.getCouponId(), couponAvailableDto.getCouponId()) &&
                        (!couponAvailableDto.getIsSimilarSuperposition() || !Objects.equals(dto.getType(), couponAvailableDto.getType())) &&
                        dto.getCouponIds().contains(couponAvailableDto.getCouponId().toString())).max(Comparator.comparing(CouponAvailableDto::getTotalRMBReduceAmount)).orElse(null);
                if (Objects.nonNull(similarDto)) {
                    couponAvailableDtoList.add(similarDto);
                }
            } else {
                // 可用叠加优惠，则查询是否有不相同类型的可叠加优惠
                CouponAvailableDto similarDto = comparingDtoList.stream().filter(couponAvailableDto -> !Objects.equals(dto.getCouponId(), couponAvailableDto.getCouponId()) &&
                        !Objects.equals(dto.getType(), couponAvailableDto.getType()) &&
                        dto.getCouponIds().contains(couponAvailableDto.getCouponId().toString())).max(Comparator.comparing(CouponAvailableDto::getTotalRMBReduceAmount)).orElse(null);
                if (Objects.nonNull(similarDto)) {
                    couponAvailableDtoList.add(similarDto);
                }
            }
            return couponAvailableDtoList;
        }
        return null;
    }

    private static void coverCouponInfoVO(CouponAvailableDto dto, CouponInfoVO infoVO) {
        BeanUtils.copyProperties(dto, infoVO);
        infoVO.setFreightReduceAmount(dto.getReduceAmount());
        infoVO.setFreightReduceCurrencyId(dto.getReduceCurrencyId());
    }

    @Override
    public List<CouponDO> getCouponList(Collection<Long> ids) {
        return couponMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<CouponDO> getCouponPage(CouponPageReqVO pageReqVO) {
        PageVO page = new PageVO();
        page.setField("c.coupon_id DESC");
        page.setRows(pageReqVO.getPageSize());
        page.setPage(pageReqVO.getPageNo());
        IPage<CouponDO> mpPage = MyBatisUtils.buildPage(page);
        couponMapper.getCouponPage(mpPage, pageReqVO);
        return PageResult.of(mpPage);
    }

    @Override
    public List<CouponDO> getCouponList(CouponExportReqVO exportReqVO) {
        return couponMapper.selectList(exportReqVO);
    }


    @Override
    public CouponRespVO getInfo(Long couponId) {
        CouponDO couponDO = couponMapper.selectById(couponId);
        CouponRespVO couponRespVO = CouponConvert.INSTANCE.convert(couponDO);
        if (StringUtils.isNotBlank(couponRespVO.getDiscountDetailed())) {
            couponRespVO.setDiscountDetailedVOs(JSONObject.parseArray(couponRespVO.getDiscountDetailed(), DiscountDetailedVO.class));
        }
        if (0 == couponRespVO.getStatus()) {
            // 草稿状态获取指定用户ID列表与指定产品ID列表
            couponRespVO.setLineIds(JSONObject.parseArray(redisHelper.get(MessageFormat.format("coupon:line:list:{0}", couponRespVO.getCouponId())), CouponLineDO.class));
            couponRespVO.setCustomerIds(JSONObject.parseArray(redisHelper.get(MessageFormat.format("coupon:customer:list:{0}", couponRespVO.getCouponId())), Long.class));
            couponRespVO.setProdIds(JSONObject.parseArray(redisHelper.get(MessageFormat.format("coupon:prod:list:{0}", couponRespVO.getCouponId())), Long.class));
        } else {
            couponRespVO.setLineIds(lineMapper.getLineIdsByCouponId(couponDO.getCouponId()));
            couponRespVO.setCustomerIds(couponCustomerService.getCustomerIdsByCouponId(couponDO.getCouponId()));
            couponRespVO.setProdIds(prodMapper.getProdIdsByCouponId(couponDO.getCouponId()));
        }
        CouponContentDO contentDO = contentMapper.selectOne(new LambdaQueryWrapperX<CouponContentDO>()
                .eq(CouponContentDO::getCouponId, couponDO.getCouponId()).last("limit 1"));
        if (contentDO != null) {
            couponRespVO.setContentEn(contentDO.getContentEn());
            couponRespVO.setContentZh(contentDO.getContentZh());
        }
        return couponRespVO;
    }

    private void verificationDiscountDetailed(Integer type, List<DiscountDetailedVO> discountDetailedVOs) {
        switch (CouponTypeEnum.valueOf(type)) {
            case FULL_REDUCE:
                for (DiscountDetailedVO vo : discountDetailedVOs) {
                    if (Objects.isNull(vo.getType())) {
                        throw exception(COUPON_FEE_TYPE_NOT_NULL);
                    }
                    if (null == vo.getFullCurrencyId() && null == vo.getProdUnit()) {
                        //  满减优惠的货币单位或者计量单位不能全为空
                        throw exception(COUPON_CURRENCY_OR_UNIT_NOT_NULL);
                    }
                    if (null == vo.getReduceCurrencyId()) {
                        throw exception(COUPON_FULL_REDUCE_CURRENCY_NOT_NULL);
                    }
                    if (null == vo.getFullAmount() || vo.getFullAmount().compareTo(BigDecimal.ZERO) < 0 ||
                            null == vo.getReduceAmount() || vo.getReduceAmount().compareTo(BigDecimal.ZERO) < 0) {
                        throw exception(COUPON_FULL_REDUCE_NOT_NULL_OR_LT_ZERO);
                    }
                    if (Objects.nonNull(vo.getFullCurrencyId()) && vo.getFullCurrencyId() > 0 && vo.getFullAmount().compareTo(vo.getReduceAmount()) < 0) {
                        throw exception(COUPON_FULL_REDUCE_VALUE_THAN_PAYMENT_VALUE);
                    }

//                    if (null != vo.getFullCurrencyId()) {
//                        if (null == vo.getFullCurrencyId() || null == vo.getReduceCurrencyId()) {
//                            throw exception(COUPON_FULL_REDUCE_CURRENCY_NOT_NULL);
//                        }
//                    } else if (null != vo.getProdUnit()) {
//                        if (null == vo.getReduceCurrencyId()) {
//                            throw exception(COUPON_FULL_REDUCE_CURRENCY_NOT_NULL);
//                        }
//                    }
//
                }
                break;
//            case SQUARE_NUMBER_FULL_REDUCE:
//                for (DiscountDetailedVO vo : discountDetailedVOs) {
//                    if (null == vo.getFullAmount() || vo.getFullAmount().compareTo(BigDecimal.ZERO) < 0 ||
//                            null == vo.getReduceAmount() || vo.getReduceAmount().compareTo(BigDecimal.ZERO) < 0) {
//                        throw exception(COUPON_FULL_REDUCE_NOT_NULL_OR_LT_ZERO);
//                    }
//                    if (null == vo.getReduceCurrencyId()) {
//                        throw exception(COUPON_FULL_REDUCE_CURRENCY_NOT_NULL);
//                    }
//                }
//                break;
            case DISCOUNT:
                for (DiscountDetailedVO vo : discountDetailedVOs) {
                    if (Objects.isNull(vo.getType())) {
                        throw exception(COUPON_FEE_TYPE_NOT_NULL);
                    }
                    if (null == vo.getFullCurrencyId() && null == vo.getProdUnit()) {
                        //  满减优惠的货币单位或者计量单位不能全为空
                        throw exception(COUPON_CURRENCY_OR_UNIT_NOT_NULL);
                    }
                    if (null == vo.getFullAmount() || vo.getFullAmount().compareTo(BigDecimal.ZERO) < 0) {
                        throw exception(COUPON_FULL_REDUCE_NOT_NULL_OR_LT_ZERO);
                    }
                    if (null == vo.getReduceAmount() || (vo.getReduceAmount().compareTo(BigDecimal.ZERO) <= 0 && vo.getReduceAmount().compareTo(BigDecimal.ONE) > 0)) {
                        throw exception(COUPON_FULL_FRACTURE_NOT_NULL_OR_LT_ZERO);
                    }

                }
                break;
            case PREFERENTIAL:
                for (DiscountDetailedVO vo : discountDetailedVOs) {
                    if (Objects.isNull(vo.getType())) {
                        throw exception(COUPON_FEE_TYPE_NOT_NULL);
                    }
                    if (null == vo.getFullCurrencyId() && null == vo.getProdUnit()) {
                        //  满减优惠的货币单位或者计量单位不能全为空
                        throw exception(COUPON_CURRENCY_OR_UNIT_NOT_NULL);
                    }
                    if (null == vo.getNetReceiptsCurrencyId()) {
                        throw exception(COUPON_FULL_REDUCE_CURRENCY_NOT_NULL);
                    }
                    if (null == vo.getFullAmount() || vo.getFullAmount().compareTo(BigDecimal.ZERO) < 0 ||
                            null == vo.getReduceAmount() || vo.getReduceAmount().compareTo(BigDecimal.ZERO) < 0) {
                        throw exception(COUPON_FULL_REDUCE_NOT_NULL_OR_LT_ZERO);
                    }
                    if (Objects.nonNull(vo.getFullCurrencyId()) && vo.getFullCurrencyId() > 0 && vo.getFullAmount().compareTo(vo.getNetReceiptsAmount()) < 0) {
                        throw exception(COUPON_FULL_NET_VALUE_THAN_PAYMENT_VALUE);
                    }
                }
                break;
            case SPECIAL_OFFER:
                for (DiscountDetailedVO vo : discountDetailedVOs) {
                    if (Objects.isNull(vo.getType())) {
                        throw exception(COUPON_FEE_TYPE_NOT_NULL);
                    }
                    if (null == vo.getReduceAmount() || vo.getReduceAmount().compareTo(BigDecimal.ZERO) < 0) {
                        throw exception(COUPON_FULL_REDUCE_NOT_NULL_OR_LT_ZERO);
                    }
                    if (Objects.isNull(vo.getProdUnit())) {
                        throw exception(COUPON_SPECIAL_OFFER_PACKING_NOT_NULL);
                    }
                    if (null == vo.getNetReceiptsCurrencyId()) {
                        throw exception(COUPON_FULL_REDUCE_CURRENCY_NOT_NULL);
                    }
                }
                break;
            case INTERVAL_PRICE:
                for (DiscountDetailedVO vo : discountDetailedVOs) {
                    if (Objects.isNull(vo.getType())) {
                        throw exception(COUPON_FEE_TYPE_NOT_NULL);
                    }
                    if (null == vo.getStartAndEndCurrencyId() && null == vo.getProdUnit()) {
                        //  满减优惠的货币单位或者计量单位不能全为空
                        throw exception(COUPON_CURRENCY_OR_UNIT_NOT_NULL);
                    }
                    if (null == vo.getStartAmount() || vo.getStartAmount().compareTo(BigDecimal.ZERO) < 0 ||
                            null == vo.getNetReceiptsAmount() || vo.getNetReceiptsAmount().compareTo(BigDecimal.ZERO) < 0) {
                        throw exception(COUPON_FULL_REDUCE_NOT_NULL_OR_LT_ZERO);
                    }
                    if (null == vo.getNetReceiptsCurrencyId()) {
                        throw exception(COUPON_FULL_REDUCE_CURRENCY_NOT_NULL);
                    }
                }
                break;
            default:
                throw exception(COUPON_TYPE_ERROR);
        }
    }


    @Override
    public List<CouponDO> getCouponSelect(CouponPageReqVO pageReqVO) {
        return couponMapper.getCouponSelect(pageReqVO, pageReqVO.getPageSize());
    }

    @Override
    public Integer countUserCoupon(Long userId) {
        long count = couponCustomerService.count(new LambdaQueryWrapper<CouponCustomerDO>()
                .eq(CouponCustomerDO::getUserId, userId)
                .eq(CouponCustomerDO::getStatus, 1));
        return Math.toIntExact(count);
    }


    @Override
    public long couponExpire() {
        return couponMapper.couponExpire(new Date());
    }

    @Override
    public List<CouponDO> getNewProdCouponList() {
        return couponMapper.getNewProdCouponList();
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateProdCoupon(Long prodId, List<Long> couponIds) {
        ProductDO productDO = productMapper.selectById(prodId);
        if (Objects.isNull(productDO)) {
            // 商品不存在
            throw exception(PRODUCT_NOT_EXISTS);
        }
        List<CouponDO> list = this.getNewProdCouponList();
        if (CollectionUtil.isNotEmpty(list)) {
            List<CouponDO> couponDOList = list.stream().filter(c -> couponIds.contains(c.getCouponId())).collect(Collectors.toList());
            if (CollectionUtil.isNotEmpty(couponDOList)) {
                for (CouponDO couponDO : couponDOList) {
                    if (couponDO.getSuitableProdType() == 0) {
                        // 优惠适用商品为全部时，不用做处理
                        continue;
                    }
                    if (couponDO.getSuitableProdType() == 1) {
                        // 优惠适用商品为指定参与时，将此品名加入
                        CouponProdDO couponProdDO = new CouponProdDO();
                        couponProdDO.setCouponId(couponDO.getCouponId());
                        couponProdDO.setProdId(prodId);
                        prodMapper.insert(couponProdDO);
                    }
                    if (couponDO.getSuitableProdType() == 2) {
                        // 优惠适用商品为指定不参与时，如果该商品指定不参与优惠则删除
                        prodMapper.delete(new LambdaQueryWrapper<CouponProdDO>()
                                .eq(CouponProdDO::getCouponId, couponDO.getCouponId())
                                .eq(CouponProdDO::getProdId, prodId));
                    }
                }
            }
            List<CouponDO> unCouponDOList = list.stream().filter(c -> !couponIds.contains(c.getCouponId())).collect(Collectors.toList());
            if (CollectionUtil.isNotEmpty(unCouponDOList)) {
                for (CouponDO couponDO : unCouponDOList) {
                    if (couponDO.getSuitableProdType() == 0) {
                        // 优惠适用商品为全部时，不能做类型改变，无法解除商品与优惠的使用绑定
                        continue;
                    }
                    if (couponDO.getSuitableProdType() == 1) {
                        // 优惠适用商品为指定参与时，如果该商品有参与优惠则删除
                        prodMapper.delete(new LambdaQueryWrapper<CouponProdDO>()
                                .eq(CouponProdDO::getCouponId, couponDO.getCouponId())
                                .eq(CouponProdDO::getProdId, prodId));
                    }
                    if (couponDO.getSuitableProdType() == 2) {
                        // 优惠适用商品为指定不参与时，将此品名加入
                        CouponProdDO couponProdDO = new CouponProdDO();
                        couponProdDO.setCouponId(couponDO.getCouponId());
                        couponProdDO.setProdId(prodId);
                        prodMapper.insert(couponProdDO);
                    }
                }
            }
        }

    }
}
