package cn.iocoder.yudao.module.product.service.product;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductDO;
import cn.iocoder.yudao.module.product.dto.ProductReqDTO;
import cn.iocoder.yudao.module.product.vo.product.*;

import javax.validation.Valid;
import java.util.List;

/**
 * 产品 Service 接口
 *
 * @author wlh
 */
public interface ProductService {

    /**
     * 创建产品
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createProduct(@Valid ProductCreateReqVO createReqVO);

    /**
     * 创建产品
     *
     * @param appProductCreateVo 创建信息
     * @return 编号
     */
    Long createAppProduct(@Valid AppProductCreateVo appProductCreateVo);

    /**
     * 更新产品
     *
     * @param updateReqVO 更新信息
     */
    void updateProduct(@Valid ProductUpdateReqVO updateReqVO);

    /**
     * 删除产品
     *
     * @param id 编号
     */
    void deleteProduct(Long id);

    /**
     * 获得产品
     *
     * @param id 编号
     * @return 产品
     */
    ProductDO getProduct(Long id);

    /**
     * 获得产品
     *
     * @param id 编号
     * @return 产品
     */
    ProductDO info(Long id);

    ProductRespVO getProductWithPriceStepClearanceList(Long id);

    /**
     * 获得产品列表
     *
     * @return 产品列表
     */
    List<ProductDO> getProductList();


    /**
     * 获得产品分页
     *
     * @param pageReqVO 分页查询
     * @return 产品分页
     */
    PageResult<ProductRespVO> getProductPage(ProductPageReqVO pageReqVO);

    PageResult<ProductRespVO> getProductPageResp(ProductReqDTO pageReqVO);

    List<ProductDO> getProductList(ProductPageReqVO pageReqVO);

    List<ProductDO> getProductListResp(ProductReqDTO pageReqVO);
    /**
     * 获得产品列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 产品列表
     */
    List<ProductDO> getProductList(ProductExportReqVO exportReqVO);

    List<ProductTypeNumVO> getProductTypeNumList(List<Long> typeIdList);

    Long typeProductCount(Long typeId);

    /**
     * 批量更新商品属性
     *
     * @param updateReqVO 请求参数
     */
    void batchUpdateProduct(@Valid ProductBatchUpdateReqVO updateReqVO);

    /**
     * 查询商品列表
     *
     * @param idList id列表
     * @return
     */
    List<ProductDO> getProductList(List<Long> idList);

    ProductDO findProdByTitle(String prodTitleZh, String prodTitleEn);

    List<ProductDO> randomTemplateProdData(Integer num);

    List<Long> getProductIdList(ProductPageReqVO pageReqVO);


}
