package cn.iocoder.yudao.module.product.service.product.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.apollo.core.event.ProductConfirmedEvent;
import cn.iocoder.yudao.framework.common.util.spring.enums.CommonStatusEnum;
import cn.iocoder.yudao.framework.common.exception.ServiceException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.dataobject.BaseDO;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.ecw.enums.ProductAuditStatusEnum;
import cn.iocoder.yudao.module.product.convert.product.ProductConvert;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductAttrDO;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductDO;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductPriceStepClearanceDO;
import cn.iocoder.yudao.module.product.dal.dataobject.productPriceSnapshot.ProductPriceSnapshotDO;
import cn.iocoder.yudao.module.product.dal.mysql.product.ProductAttrMapper;
import cn.iocoder.yudao.module.product.dal.mysql.product.ProductMapper;
import cn.iocoder.yudao.module.product.dal.mysql.product.ProductPriceStepClearanceMapper;
import cn.iocoder.yudao.module.product.dal.mysql.productPriceSnapshot.ProductPriceSnapshotMapper;
import cn.iocoder.yudao.module.product.dto.ProductReqDTO;
import cn.iocoder.yudao.module.product.service.coupon.CouponService;
import cn.iocoder.yudao.module.product.service.product.ProductPriceService;
import cn.iocoder.yudao.module.product.service.product.ProductPriceStepClearanceService;
import cn.iocoder.yudao.module.product.service.product.ProductService;
import cn.iocoder.yudao.module.product.vo.product.*;
import cn.iocoder.yudao.module.product.vo.productPrice.ClearancePriceEvent;
import cn.iocoder.yudao.module.system.api.user.AdminUserApi;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.google.common.base.Joiner;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.ecw.enums.ErrorCodeConstants.PRODUCT_NOT_EXISTS;
import static java.util.stream.Collectors.toList;

/**
 * 产品 Service 实现类
 *
 * @author wlh
 */
@Service
@Validated
public class ProductServiceImpl implements ProductService {

    @Resource
    private ProductMapper productMapper;
    @Resource
    private ProductAttrMapper productAttrMapper;
    @Resource
    @Lazy
    private CouponService couponService;

    @Resource
    private ProductPriceStepClearanceMapper productPriceStepClearanceMapper;

    @Resource
    private ProductPriceStepClearanceService productPriceStepClearanceService;

    @Resource
    @Lazy
    private ProductPriceService productPriceService;

    @Resource
    private ApplicationContext applicationContext;


    @Resource
    private AdminUserApi adminUserApi;

    @Resource
    private ProductPriceSnapshotMapper productPriceSnapshotMapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createProduct(ProductCreateReqVO createReqVO) {
        // 插入
        ProductDO product = ProductConvert.INSTANCE.convert(createReqVO);
        checkUnique(product);

        productMapper.insert(product);

        //插入清关费
        List<ProductPriceStepClearanceDO> priceStepClearanceList = createReqVO.getPriceStepClearanceList();
        if (CollectionUtil.isNotEmpty(priceStepClearanceList)) {
            priceStepClearanceList.stream().peek(t -> t.setProductId(product.getId())).forEach(productPriceStepClearanceDO -> {
                productPriceStepClearanceMapper.insert(productPriceStepClearanceDO);
            });
        }

        //模板复制
        if (Boolean.TRUE.equals(createReqVO.getIsCopyProductPriceTemplate()) && createReqVO.getProductTemplateId() != null) {
            // 专线空运
            productPriceService.productPriceCopy("3", createReqVO.getProductTemplateId(), CollectionUtil.newArrayList(product.getId()));
            // // 海空联运
            // productPriceService.productPriceCopy("4", updateReqVO.getProductTemplateId(), CollectionUtil.newArrayList(updateReqVO.getId()));
        }

        // 判断是否对商品适用优惠做了调整
        if (CollectionUtil.isNotEmpty(createReqVO.getCouponIds())) {
            couponService.updateProdCoupon(product.getId(), createReqVO.getCouponIds());
        }
        // 返回
        return product.getId();
    }

    private void checkUnique(ProductDO productDO) {
        String titleZh = productDO.getTitleZh();
        String titleEn = productDO.getTitleEn();

        Long id = productDO.getId();
        if(id != null) {
            //更新
            ProductDO dbDO = getProduct(id);
            //校验名称有没有修改
            if(dbDO.getTitleZh().equals(titleZh) && dbDO.getTitleEn().equals(titleEn)) return;
        }

        List<ProductDO> productList = productMapper.selectList(new LambdaQueryWrapperX<ProductDO>()
            .and(i -> i.eq(ProductDO::getTitleZh, titleZh)
                .or()
                    .eq(ProductDO::getTitleEn, titleEn)
            )
                .ne(id != null, ProductDO::getId, id)
                .eq(ProductDO::getStatus, 0)
        );
        if(CollectionUtil.isNotEmpty(productList)) {
            ProductDO enProduct = productList.stream().filter(t -> t.getTitleEn().equals(titleEn)).findFirst().orElse(null);
            String msg = "商品中文名称已存在，请勿重复添加";
            if(enProduct != null) {
                msg = "商品英文名称已存在，请勿重复添加";
            }
            throw new ServiceException(500, msg);

        }
    }

    @Override
    public Long createAppProduct(AppProductCreateVo appProductCreateVo) {
        ProductDO product = new ProductDO();
        product.setTypeId(appProductCreateVo.getTypeId());
        product.setAttrId(appProductCreateVo.getAttrId());
        product.setTitleZh(appProductCreateVo.getTitleZh());
        product.setTitleEn(appProductCreateVo.getTitleEn());

        checkUnique(product);

        product.setStatus(1);
        productMapper.insert(product);
        return product.getId();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateProduct(ProductUpdateReqVO updateReqVO) {
        // 校验存在
        Long prodId = updateReqVO.getId();
        this.validateProductExists(prodId);
        ProductDO productDO = productMapper.selectById(prodId);

        BigDecimal airWeightLimitNew = updateReqVO.getAirWeightLimit();
        BigDecimal airWeightLimitOld = productDO.getAirWeightLimit();
        // 更新
        ProductDO updateObj = ProductConvert.INSTANCE.convert(updateReqVO);

        checkUnique(updateObj);

        productMapper.updateById(updateObj);

        // //自动处理商品超重异常
        // ProductUpdateAutoProcessOverWeightExceptionEvent event = new ProductUpdateAutoProcessOverWeightExceptionEvent(prodId, airWeightLimitOld, airWeightLimitNew);
        // applicationContext.publishEvent(event);

        //添加修改日志
        batchAddSnapshot(Collections.singletonList(updateObj));

        //更新清关费
        List<ProductPriceStepClearanceDO> priceStepClearanceList = updateReqVO.getPriceStepClearanceList();
        clearAndSavePriceStepClearanceList(Collections.singletonList(prodId), priceStepClearanceList);

        //模板复制
        if (Boolean.TRUE.equals(updateReqVO.getIsCopyProductPriceTemplate()) && updateReqVO.getProductTemplateId() != null) {
            // 专线空运
            productPriceService.productPriceCopy("3", updateReqVO.getProductTemplateId(), CollectionUtil.newArrayList(prodId));
            // // 海空联运
            // productPriceService.productPriceCopy("4", updateReqVO.getProductTemplateId(), CollectionUtil.newArrayList(updateReqVO.getId()));
        }

        // 判断是否对商品适用优惠做了调整
        if (CollectionUtil.isNotEmpty(updateReqVO.getCouponIds())) {
            couponService.updateProdCoupon(updateObj.getId(), updateReqVO.getCouponIds());
        }
        //如果是是商品新增异常带过来的则需要进行处理
        if (null != updateReqVO.getProductId()) {
            applicationContext.publishEvent(new ProductConfirmedEvent(updateReqVO.getProductId()));
        }

    }

    private void clearAndSavePriceStepClearanceList(List<Long> productIdList, List<ProductPriceStepClearanceDO> stepList) {

        if (CollectionUtil.isEmpty(productIdList)) return;

        productPriceStepClearanceMapper.physicsDelete(Joiner.on(",").join(productIdList));

        if (CollectionUtil.isNotEmpty(stepList)) {
            List<ProductPriceStepClearanceDO> insertList = new ArrayList<>();
            for (Long productId : productIdList) {

                //通知异常处理清关费
                applicationContext.publishEvent(new ClearancePriceEvent(productId));

                stepList.forEach(t -> {
                    ProductPriceStepClearanceDO step = ObjectUtil.clone(t);
                    step.setId(null);
                    step.setProductId(productId);
                    Date date = new Date();
                    if (step.getCreateTime() == null) {
                        step.setCreateTime(date);
                    }
                    step.setUpdateTime(date);
                    step.setDeleted(false);
                    insertList.add(step);
                });
            }
            productPriceStepClearanceMapper.insertBatchSomeColumn(insertList);
        }
    }


    /**
     * 批量添加快照日志
     *
     * @param productDOList 快照列表
     */
    private void batchAddSnapshot(List<ProductDO> productDOList) {
        if (CollectionUtil.isEmpty(productDOList)) return;

        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        String creatorName = adminUserApi.getUserName(String.valueOf(loginUserId));

        int batchCount = 500;
        //sql长度有限制，分批次
        int countBatch = (int) Math.ceil((double) productDOList.size() / batchCount);
        List<List<ProductDO>> splitList = Stream.iterate(0, n -> n + 1)
                .limit(countBatch)
                .parallel()
                .map(a -> productDOList.stream().skip(a * batchCount).limit(batchCount).parallel().collect(toList())
                ).collect(toList());
        for (List<ProductDO> productList : splitList) {

            List<ProductPriceSnapshotDO> priceSnapshotList = new ArrayList<>();
            for (ProductDO productDO : productList) {

                Long productId = productDO.getId();

                List<ProductPriceStepClearanceDO> productPriceStepClearanceDOList = productPriceStepClearanceMapper.selectList(new LambdaQueryWrapperX<ProductPriceStepClearanceDO>()
                        .eq(ProductPriceStepClearanceDO::getProductId, productId).eq(BaseDO::getDeleted, Boolean.FALSE));

                if (CollectionUtil.isNotEmpty(productPriceStepClearanceDOList)) {

                    ProductPriceSnapshotDO snapshotDO = new ProductPriceSnapshotDO();
                    snapshotDO.setProductId(productId);
                    snapshotDO.setCreatorName(creatorName);

                    String content = JSON.toJSONString(productPriceStepClearanceDOList);
                    snapshotDO.setContent(content);
                    snapshotDO.setDeleted(false);
                    priceSnapshotList.add(snapshotDO);
                }
            }
            if (CollectionUtil.isNotEmpty(priceSnapshotList)) {
                productPriceSnapshotMapper.insertBatchSomeColumn(priceSnapshotList);
            }
        }
    }

    @Override
    public void deleteProduct(Long id) {
        // 校验存在
        this.validateProductExists(id);
        // 删除
        productMapper.deleteById(id);

        //删除商品清关费阶梯价格
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("product_id", id);
        productPriceStepClearanceMapper.deleteByMap(paramMap);
    }

    private void validateProductExists(Long id) {
        if (productMapper.selectById(id) == null) {
            throw exception(PRODUCT_NOT_EXISTS);
        }
    }

    @Override
    public ProductDO getProduct(Long id) {
        return productMapper.selectById(id);
    }

    @Override
    public ProductDO info(Long id) {
        ProductDO productDO = productMapper.selectById(id);
        if (Objects.nonNull(productDO) && StringUtils.isNotBlank(productDO.getAttrId())) {
            List<Long> attrIdList = Arrays.stream(productDO.getAttrId().split(StrUtil.COMMA)).map(Long::parseLong).collect(Collectors.toList());
            List<ProductAttrDO> attrList = productAttrMapper.selectList(new LambdaQueryWrapper<ProductAttrDO>().in(ProductAttrDO::getId, attrIdList));
            if (CollectionUtil.isNotEmpty(attrList)) {
                productDO.setAttrList(attrList);
            }
        }
        return productDO;
    }

    @Override
    public ProductRespVO getProductWithPriceStepClearanceList(Long id) {
        ProductDO productDO = productMapper.selectById(id);
        ProductRespVO productRespVO = ProductConvert.INSTANCE.convert(productDO);
        if (productRespVO != null) {
            List<ProductPriceStepClearanceBackVO> productPriceStepClearanceList = productPriceStepClearanceService.getProductPriceStepClearanceList(productRespVO.getId());
            productRespVO.setPriceStepClearanceList(productPriceStepClearanceList);
        }
        return productRespVO;
    }

    @Override
    public List<ProductDO> getProductList() {
        //查询已审核，已上架的商品
        return productMapper.selectList(
                new LambdaQueryWrapperX<ProductDO>()
                        .eq(ProductDO::getStatus, CommonStatusEnum.ENABLE.getStatus())
                        .eq(ProductDO::getAuditStatus, ProductAuditStatusEnum.ENABLE.getStatus())
        );
    }



    @Override
    public List<ProductDO> getProductList(List<Long> idList) {
        return productMapper.selectList(
                new LambdaQueryWrapperX<ProductDO>()
                        .inIfPresent(ProductDO::getId, idList)
//                        .eq(ProductDO::getStatus, CommonStatusEnum.ENABLE.getStatus())
//                        .eq(ProductDO::getAuditStatus, ProductAuditStatusEnum.ENABLE.getStatus())
        );
    }

    @Override
    public PageResult<ProductRespVO> getProductPage(ProductPageReqVO pageReqVO) {
        if (pageReqVO.isFilter()) {
            pageReqVO.setAuditStatus(2);
        }

        IPage<ProductDO> mpPage = MyBatisUtils.buildPage(pageReqVO);
        productMapper.selectPage2(mpPage, pageReqVO);

        PageResult<ProductDO> pageResult = PageResult.of(mpPage);
        PageResult<ProductRespVO> productRespVOPageResult = ProductConvert.INSTANCE.convertPage(pageResult);

        List<ProductRespVO> list = productRespVOPageResult.getList();
        if (CollectionUtil.isNotEmpty(list)) {
            for (ProductRespVO productRespVO : list) {
                List<ProductPriceStepClearanceBackVO> productPriceStepClearanceList = productPriceStepClearanceService.getProductPriceStepClearanceList(productRespVO.getId());
                productRespVO.setPriceStepClearanceList(productPriceStepClearanceList);
                productRespVO.setPriceStepClearanceCount(new Long(productPriceStepClearanceList.size()));
            }
        }
        return productRespVOPageResult;
    }


    @Override
    public PageResult<ProductRespVO> getProductPageResp(ProductReqDTO pageReqVO) {
        if (pageReqVO.isFilter()) {
            pageReqVO.setAuditStatus(2);
        }

        IPage<ProductDO> mpPage = MyBatisUtils.buildPage(pageReqVO);
        productMapper.selectPageResp(mpPage, pageReqVO);

        PageResult<ProductDO> pageResult = PageResult.of(mpPage);
        PageResult<ProductRespVO> productRespVOPageResult = ProductConvert.INSTANCE.convertPage(pageResult);

        List<ProductRespVO> list = productRespVOPageResult.getList();
        if (CollectionUtil.isNotEmpty(list)) {
            for (ProductRespVO productRespVO : list) {
                List<ProductPriceStepClearanceBackVO> productPriceStepClearanceList = productPriceStepClearanceService.getProductPriceStepClearanceList(productRespVO.getId());
                productRespVO.setPriceStepClearanceList(productPriceStepClearanceList);
                productRespVO.setPriceStepClearanceCount(new Long(productPriceStepClearanceList.size()));
            }
        }
        return productRespVOPageResult;
    }

    @Override
    public List<Long> getProductIdList(ProductPageReqVO pageReqVO) {
        if (pageReqVO.isFilter()) {
            pageReqVO.setAuditStatus(2);
        }
        List<ProductDO> productDOList = productMapper.selectIdList(pageReqVO);

        return productDOList.stream().map(ProductDO::getId).collect(toList());
    }

    @Override
    public List<ProductDO> getProductList(ProductPageReqVO pageReqVO) {
        if (pageReqVO.isFilter()) {
            pageReqVO.setAuditStatus(2);
        }
        return productMapper.selectList(pageReqVO);
    }

    @Override
    public List<ProductDO> getProductListResp(ProductReqDTO pageReqVO) {
        if (pageReqVO.isFilter()) {
            pageReqVO.setAuditStatus(2);
        }
        return productMapper.selectListReq(pageReqVO);
    }

    @Override
    public List<ProductDO> getProductList(ProductExportReqVO exportReqVO) {
        if (exportReqVO.isFilter()) {
            exportReqVO.setAuditStatus(2);
        }
        return productMapper.selectList(exportReqVO);
    }

    @Override
    public List<ProductTypeNumVO> getProductTypeNumList(List<Long> typeIdList) {
        return productMapper.getProductTypeNumList(typeIdList);
    }

    @Override
    public Long typeProductCount(Long typeId) {
        return productMapper.selectCount("type_id", typeId);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void batchUpdateProduct(@Valid ProductBatchUpdateReqVO updateReqVO) {
        //修改所有
        ProductDO productDO = new ProductDO();
        BeanUtils.copyProperties(updateReqVO, productDO);
        if (updateReqVO.getIsAllProduct() == 1) {
            productMapper.update(productDO, new LambdaQueryWrapperX<ProductDO>()
                    .eq(ProductDO::getStatus, CommonStatusEnum.ENABLE.getStatus())
                    .eq(ProductDO::getAuditStatus, ProductAuditStatusEnum.ENABLE.getStatus())
            );

            List<ProductDO> productDOList = productMapper.selectList(new LambdaQueryWrapperX<ProductDO>()
                    .eq(ProductDO::getStatus, CommonStatusEnum.ENABLE.getStatus())
                    .eq(ProductDO::getAuditStatus, ProductAuditStatusEnum.ENABLE.getStatus()));
            if (CollectionUtil.isNotEmpty(productDOList)) {
                // 记录日志
                batchAddSnapshot(productDOList);

                //更新
                List<Long> productIds = productDOList.stream().map(ProductDO::getId).collect(toList());
                clearAndSavePriceStepClearanceList(productIds, updateReqVO.getPriceStepClearanceList());
            }

        } else {
            List<Long> idList = updateReqVO.getIdList();
            if (CollectionUtil.isEmpty(idList)) throw new IllegalArgumentException("idList不能为空");
            productMapper.update(productDO, new LambdaQueryWrapperX<ProductDO>()
                    .in(ProductDO::getId, idList)
            );


            List<ProductDO> productDOList = productMapper.selectList(new LambdaQueryWrapperX<ProductDO>()
                    .in(ProductDO::getId, idList));
            if (CollectionUtil.isNotEmpty(productDOList)) {
                // 记录日志
                batchAddSnapshot(productDOList);

                //更新
                List<Long> productIds = productDOList.stream().map(ProductDO::getId).collect(toList());
                clearAndSavePriceStepClearanceList(productIds, updateReqVO.getPriceStepClearanceList());
            }
        }
    }

    @Override
    public ProductDO findProdByTitle(String prodTitleZh, String prodTitleEn) {
        return productMapper.selectOne(new LambdaQueryWrapper<ProductDO>()
                .eq(ProductDO::getTitleZh, prodTitleZh)
                .or(w -> w.eq(ProductDO::getTitleEn, prodTitleEn))
                .last("limit 1"));
    }

    @Override
    public List<ProductDO> randomTemplateProdData(Integer num) {
        return productMapper.randomTemplateProdData(num);
    }
}
