package cn.iocoder.yudao.module.product.vo.coupon;

import com.fasterxml.jackson.annotation.JsonFormat;
import lombok.*;

import java.math.BigDecimal;
import java.util.*;
import io.swagger.annotations.*;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import org.springframework.format.annotation.DateTimeFormat;

import static cn.iocoder.yudao.framework.common.util.date.DateUtils.FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND;

@ApiModel("管理后台 - 优惠券信息分页 Request VO")
@Data
@EqualsAndHashCode(callSuper = true)
@ToString(callSuper = true)
public class CouponPageReqVO extends PageParam {

    @ApiModelProperty(value = "标题-中文")
    private String titleZh;

    @ApiModelProperty(value = "标题-英文")
    private String titleEn;

    @ApiModelProperty(value = "类型 1:优惠卷 2:金额-满减 3:方数-满减 4 折扣 5优惠 6 特价 7区间价")
    private Integer type;

    @ApiModelProperty(value = "使用条件(优惠卷专用)")
    private BigDecimal cashCondition;

    @ApiModelProperty(value = "减免金额(优惠卷专用)")
    private BigDecimal reduceAmount;

    @ApiModelProperty(value = "条件货币ID")
    private Integer conditionCurrencyId;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @JsonFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "开始开始时间")
    private Date beginStartTime;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @JsonFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "结束开始时间")
    private Date endStartTime;

    @ApiModelProperty(value = "减免货币ID")
    private Integer reduceCurrencyId;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @JsonFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "开始结束时间")
    private Date beginEndTime;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @JsonFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "结束结束时间")
    private Date endEndTime;

    @ApiModelProperty(value = "领券后X天起生效")
    private Integer afterReceiveDays;

    @ApiModelProperty(value = "有效天数")
    private Integer validDays;

    @ApiModelProperty(value = "费用类型：1 运输费 2 清关费 3 总费用(优惠卷专用)")
    private Integer costType;

    @ApiModelProperty(value = "优惠清单")
    private String discountDetailed;

    @ApiModelProperty(value = "同类优惠劵是否可以叠加 0否  1是")
    private Boolean isSimilarSuperposition;

    @ApiModelProperty(value = "可叠加优惠劵列表")
    private String couponIds;

    @ApiModelProperty(value = "适用用户类型 0全部用户 1部分用户 2指定用户不参与 3 注册即送")
    private Integer suitableUserType;

    @ApiModelProperty(value = "适用线路类型 0全部线路参与 1指定线路参与 2指定线路不参与")
    private Integer suitableLineType;

    @ApiModelProperty(value = "适用产品类型 0全部产品参与 1指定产品参与 2指定产品不参与")
    private Integer suitableProdType;

    @ApiModelProperty(value = "每个用户领券上限，如不填则默认为1")
    private Integer limitNum;

    @ApiModelProperty(value = "优惠券过期状态 0:过期 1:未过期")
    private Boolean overdueStatus;

    @ApiModelProperty(value = "优惠券投放状态(-1:取消投放 0:等待投放 1:投放 2:违规下架 3:等待审核)")
    private Integer putonStatus;

    @ApiModelProperty(value = "获取方式 0=客户领取 1=平台发放")
    private Integer getWay;

    @ApiModelProperty(value = "订单属性(可多选) 1 普货 2重货 3泡货")
    private String orderAttr;

    @ApiModelProperty(value = "是否发送站内信 0否  1是")
    private Boolean isSiteContent;

    @ApiModelProperty(value = "品牌(可多选) 1 已备案 2 中性 3 未备案")
    private String brand;

    @ApiModelProperty(value = "是否控货(可多选) 0否 1是")
    private String isCargoControl;

    @ApiModelProperty(value = "单证报关(可多选) 1 我司全代 2自单代报 3混合报关")
    private String documentDeclaration;

    @ApiModelProperty(value = "优惠券状态(0:草稿 1:发布)")
    private Integer status;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "开始创建时间")
    private Date beginCreateTime;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "结束创建时间")
    private Date endCreateTime;

    @ApiModelProperty(value = "关键字")
    private String searchKey;

}
