package cn.iocoder.yudao.module.product.controller.admin.product;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.dict.core.dto.DictDataRespDTO;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.module.product.convert.product.ProductConvert;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductAttrDO;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductDO;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductTypeDO;
import cn.iocoder.yudao.module.product.dto.ProductReqDTO;
import cn.iocoder.yudao.module.product.enums.AuditEnum;
import cn.iocoder.yudao.module.product.service.product.ProductAttrService;
import cn.iocoder.yudao.module.product.service.product.ProductPriceStepClearanceService;
import cn.iocoder.yudao.module.product.service.product.ProductService;
import cn.iocoder.yudao.module.product.service.product.ProductTypeService;
import cn.iocoder.yudao.module.product.vo.product.*;
import cn.iocoder.yudao.module.product.vo.productAttr.ProductAttrExportReqVO;
import cn.iocoder.yudao.module.product.vo.productPrice.ProductPricePageReqVO;
import cn.iocoder.yudao.module.product.vo.productType.ProductTypeExportReqVO;
import cn.iocoder.yudao.module.system.api.dict.DictDataApi;
import cn.iocoder.yudao.module.system.api.file.FileMakeApi;
import cn.iocoder.yudao.module.system.api.file.dto.FileMakeReqDTO;
import cn.iocoder.yudao.module.system.api.user.AdminUserApi;
import cn.iocoder.yudao.module.system.enums.download.DownloadTypeEnum;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.google.common.base.Joiner;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;

import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import java.util.stream.Collectors;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

import cn.iocoder.yudao.framework.excel.util.ExcelUtils;

import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;


@Api(tags = "管理后台 - 商品")
@RestController
@RequestMapping("/ecw/product")
@Validated
public class ProductController {

    @Resource
    private ProductService productService;

    @Resource
    private DictDataApi dictDataApi;

    @Resource
    private ProductPriceStepClearanceService productPriceStepClearanceService;
    @Resource
    private ProductTypeService productTypeService;
    @Resource
    private ProductAttrService productAttrService;
    @Resource
    private AdminUserApi adminUserApi;

    @Resource
    private FileMakeApi fileMakeApi;

    @PostMapping("/create")
    @ApiOperation("创建商品")
    @PreAuthorize("@ss.hasPermission('ecw:product:create')")
    public CommonResult<Long> createProduct(@Valid @RequestBody ProductCreateReqVO createReqVO) {
        return success(productService.createProduct(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新商品")
    @PreAuthorize("@ss.hasPermission('ecw:product:update')")
    public CommonResult<Boolean> updateProduct(@Valid @RequestBody ProductUpdateReqVO updateReqVO) {
        productService.updateProduct(updateReqVO);
        return success(true);
    }

    @PutMapping("/batchUpdate")
    @ApiOperation("批量更新商品")
    @PreAuthorize("@ss.hasPermission('ecw:product:update')")
    public CommonResult<Boolean> batchUpdateProduct(@Valid @RequestBody ProductBatchUpdateReqVO updateReqVO) {
        productService.batchUpdateProduct(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除商品")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:product:delete')")
    public CommonResult<Boolean> deleteProduct(@RequestParam("id") Long id) {
        productService.deleteProduct(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得商品")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    // @PreAuthorize("@ss.hasPermission('ecw:product:query')")
    public CommonResult<ProductRespVO> getProduct(@RequestParam("id") Long id) {
        return success(productService.getProductWithPriceStepClearanceList(id));
    }

    @GetMapping("/list")
    @ApiOperation("获得商品列表")
    // @PreAuthorize("@ss.hasPermission('ecw:product:query')")
    public CommonResult<List<ProductRespVO>> getProductList(ProductExportReqVO req) {
        List<ProductDO> list = productService.getProductList(req);
        List<ProductRespVO> productList = ProductConvert.INSTANCE.convertList(list);
//        if(CollectionUtil.isNotEmpty(productList)) {
//            for (ProductRespVO productRespVO : productList) {
//                productRespVO.setCreatorName(adminUserApi.getUserName(productRespVO.getCreator()));
//                productRespVO.setUpdaterName(adminUserApi.getUserName(productRespVO.getUpdater()));
//            }
//        }
        return success(productList);
    }

    @GetMapping("/page")
    @ApiOperation("获得商品分页")
    // @PreAuthorize("@ss.hasPermission('ecw:product:query')")
    public CommonResult<PageResult<ProductRespVO>> getProductPage(@Valid ProductReqDTO pageVO) {
        //设置有效的transportTypeList
        initDictParamsCopy(pageVO);
        PageResult<ProductRespVO> pageResult = productService.getProductPageResp(pageVO);
        //PageResult<ProductRespVO> pageResult = productService.getProductPage(pageVO);

        // List<ProductRespVO> productList = page.getList();
        // if(CollectionUtil.isNotEmpty(productList)) {
        //     for (ProductRespVO productRespVO : productList) {
        //         productRespVO.setCreatorName(adminUserApi.getUserName(productRespVO.getCreator()));
        //         productRespVO.setUpdaterName(adminUserApi.getUserName(productRespVO.getUpdater()));
        //     }
        // }
        return success(pageResult);
    }

    private void initDictParams(ProductPageReqVO pageParam) {
        // List<DictDataRespDTO> dictList = dictDataApi.getDictDatas("transport_type");
        // List<String> existTypeList = dictList.stream()
        //         .map(DictDataRespDTO::getValue)
        //         .collect(Collectors.toList());
        // if(CollectionUtil.isNotEmpty(existTypeList)) {
        //     pageParam.setEnableTransportTypeList(existTypeList);
        // }

        //目前只复制空运模板
        pageParam.setEnableTransportTypeList(CollectionUtil.newArrayList("3"));
    }


    private void initDictParamsCopy(ProductReqDTO pageParam) {

        //目前只复制空运模板
        pageParam.setEnableTransportTypeList(CollectionUtil.newArrayList("3"));
    }

    @PostMapping("/add")
    @ApiOperation("添加商品")
    public CommonResult<Long> getProductPage(@Valid @RequestBody AppProductCreateVo appProductCreateVo) {
        return success(productService.createAppProduct(appProductCreateVo));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出商品 Excel")
    @PreAuthorize("@ss.hasPermission('ecw:product:export')")
    @OperateLog(type = EXPORT)
    public CommonResult<Boolean> exportProductExcel(@Valid ProductReqDTO exportReqVO,
              HttpServletResponse response) throws IOException {

        FileMakeReqDTO reqDTO = new FileMakeReqDTO();
        reqDTO.setType(DownloadTypeEnum.PRODUCT_EXCEL_EXPORT.getType());
        reqDTO.setName("商品导出Excel");
        reqDTO.setFileSuffix("xlsx");
        reqDTO.setTemporaryFile(true);
        reqDTO.setUserType(2);
        reqDTO.setLang(I18nMessage.getLang());
        reqDTO.setRequestParams(JSONObject.toJSONString(exportReqVO));
        fileMakeApi.sendFileMake(reqDTO);
        return CommonResult.success(true);
    }


    @GetMapping("/simpleList")
    @ApiOperation("获得商品列表，返回简洁信息")
    // @PreAuthorize("@ss.hasPermission('ecw:product:query')")
    public CommonResult<List<ProductRespVO>> getProductList(@Valid ProductPageReqVO pageVO) {
        List<ProductDO> resultList = productService.getProductList(pageVO);
        List<ProductRespVO> productList = ProductConvert.INSTANCE.convertList(resultList);
        return success(productList);
    }

    @GetMapping("/simpleIdList")
    @ApiOperation("获得商品列表，返回商品ID列表")
    public CommonResult<List<Long>> getProductIdList(@Valid ProductPageReqVO pageVO) {
        List<Long> resultList = productService.getProductIdList(pageVO);
        return success(resultList);
    }


    @GetMapping("/priceStepClearanceList")
    @ApiOperation("获得商品清关费阶梯价格列表")
    // @PreAuthorize("@ss.hasPermission('ecw:product:query')")
    public CommonResult<List<ProductPriceStepClearanceBackVO>> getProductList(@RequestParam("productId") Long productId) {
        List<ProductPriceStepClearanceBackVO> list = productPriceStepClearanceService.getProductPriceStepClearanceList(productId);
        return success(list);
    }

}
