package cn.iocoder.yudao.module.product.controller.app.product;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.dict.core.dto.DictDataRespDTO;
import cn.iocoder.yudao.framework.security.core.annotations.PreAuthenticated;
import cn.iocoder.yudao.module.product.convert.product.ProductConvert;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductDO;
import cn.iocoder.yudao.module.product.service.product.ProductService;
import cn.iocoder.yudao.module.product.vo.product.ProductExportReqVO;
import cn.iocoder.yudao.module.product.vo.product.ProductPageReqVO;
import cn.iocoder.yudao.module.product.vo.product.ProductRespVO;
import cn.iocoder.yudao.module.system.api.dict.DictDataApi;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;


@Api(tags = "app-web - 商品")
@RestController
@RequestMapping("/ecw/product")
@Validated
public class AppProductController {

    @Resource
    private ProductService productService;

    @Resource
    private DictDataApi dictDataApi;


    @GetMapping("/get")
    @ApiOperation("获得商品")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthenticated
    public CommonResult<ProductRespVO> getProduct(@RequestParam("id") Long id) {
        return success(productService.getProductWithPriceStepClearanceList(id));
    }

    @GetMapping("/list")
    @ApiOperation("获得商品列表")
    @PreAuthenticated
    public CommonResult<List<ProductRespVO>> getProductList(ProductExportReqVO req) {
        List<ProductDO> list = productService.getProductList(req);
        return success(ProductConvert.INSTANCE.convertList(list));
    }


    private void initDictParams(ProductPageReqVO pageParam) {
        // List<DictDataRespDTO> dictList = dictDataApi.getDictDatas("transport_type");
        // List<String> existTypeList = dictList.stream()
        //         .map(DictDataRespDTO::getValue)
        //         .collect(Collectors.toList());
        // if(CollectionUtil.isNotEmpty(existTypeList)) {
        //     pageParam.setEnableTransportTypeList(existTypeList);
        // }

        //目前只复制空运模板
        pageParam.setEnableTransportTypeList(CollectionUtil.newArrayList("3"));
    }


    @GetMapping("/page")
    @ApiOperation("获得商品分页")
    @PreAuthenticated
    public CommonResult<PageResult<ProductRespVO>> getProductPage(@Valid ProductPageReqVO pageVO) {
        initDictParams(pageVO);
        return success(productService.getProductPage(pageVO));
    }


}
